#!/usr/bin/php
<?php
/**
 * zForum Migration Script
 * This script patch migration from version 1.0.8 to 1.1.0 running on PostgeSQL 8.0.x
 * @author Marcin F. Wisniowski <marcin.wisniowski@mfw.pl>
 * @version 1.0.0
 */
/////////////////////////////////////////////////////////////////////
// Configure Section
// Edit this variable to open MySQL DB connection
$db_host = 'localhost';
$db_user = 'postgres';
$db_pass = 'postgres';
$db_name = 'zforum';
$db_port = '5432';
$new_line_str = '<br />';
// Do not edit below this line if you don't know what are you doing
/////////////////////////////////////////////////////////////////////
$db_link = null;
/////////////////////////////////////////////////////////////////////
// Migration process starts here
connectToDB();
print $new_line_str.'DB Migration process starts'.$new_line_str;
// 1. Add cat_order field into zf_forum_category table and fill with default values
updateCategoryTable();
print 'Phase 1 - Update Category table: Done'.$new_line_str;
// 2. Split zf_topic table into zf_topics and zf_posts
createTopicsAndPostsTable();
splitTopicTable();
print 'Phase 2 - Split Topic table: Done'.$new_line_str;
// 3. Patch zf_forum table
patchForumTable();
print 'Phase 3 - Update forum table: Done'.$new_line_str;
// 4. Patch System properties
patchSystemProperies();
print 'Phase 4 - Add System Properties: Done'.$new_line_str;
// 5. Delete Table zf_topic (old-one)
deleteTopicTable();
print 'Phase 5 - Delete zf_topic table: Done'.$new_line_str;
closeConnectionToDB();
// Migration process ends
/////////////////////////////////////////////////////////////////////
function connectToDB() {
    global $db_host, $db_user, $db_pass, $db_name, $db_link, $db_port;
    $connection_string = "host='$db_host' port='$db_port' dbname='$db_name' user='$db_user' password='$db_pass'";
    $db_link = @pg_connect($connection_string) or die ('Can\'t connect to DB Server');

}
function closeConnectionToDB() {
	global $db_link;
	@pg_close($db_link);
}
function runQuery($query) {
    global $db_link;
    $result = @pg_query($db_link, $query) or die('Query failed: '.$query.$new_line_str.pg_last_error($db_link) );
    return $result;
}
function fetchAssoc($query_result) {
    return @pg_fetch_array($query_result, NULL, PGSQL_ASSOC);
}
function updateCategoryTable() {
    $sql = 'ALTER TABLE zf_forum_category ADD COLUMN cat_order int2 NOT NULL DEFAULT 0;';
    runQuery($sql);
    $sql = 'SELECT cat_id FROM zf_forum_category;';
    $result = runQuery($sql);
    $order = 1;
    while ($line = fetchAssoc($result)) {
        $sql = "UPDATE zf_forum_category SET cat_order = $order WHERE cat_id = $cat_id;";
        $order++;
    }
}
function createTopicsAndPostsTable() {
    $sql = 'create table zf_topics (
                topic_id serial unique primary key not null,
                forum_id integer not null references zf_forum(forum_id) on delete cascade,
                topic_title varchar(128) not null,
                topic_desc varchar(255),
                creation_user varchar(40) not null,
                creation_date timestamp not null,
                topic_views integer not null default 0,
                topic_replies integer not null default 0,
                topic_first_post_id integer not null default 0,
                topic_last_post_id integer not null default 0,
                locked_flag int2 not null default 0,
                disabled_flag int2 not null default 0,
                sticky_flag int2 not null default 0,
                poll_flag int2 not null default 0,
                system_announcement_flag int2 not null default 0
                );';
    runQuery($sql);
    $sql = 'create index zt_creation_date on zf_topics(creation_date);';
    runQuery($sql);
    $sql = 'create index zt_system_announcement_flag on zf_topics(system_announcement_flag);';
    runQuery($sql);
    $sql = 'ALTER TABLE zf_polls DROP CONSTRAINT zf_polls_topic_id_fkey;';
    runQuery($sql);
    $sql = 'ALTER TABLE zf_polls ADD CONSTRAINT zf_polls_topic_id_fkey FOREIGN KEY (topic_id) REFERENCES zf_topics (topic_id) ON UPDATE NO ACTION ON DELETE CASCADE;';
    $sql = 'create table zf_posts (
                post_id serial unique primary key not null,
                topic_id integer not null references zf_topics(topic_id) on delete cascade,
                content text not null,
                creation_user varchar(40) not null,
                creation_date timestamp not null,
                modifying_user varchar(40) not null,
                modifying_date timestamp not null,
                ip_address varchar(15)
                );';
    runQuery($sql);
    $sql = 'create index zfp_creation_date on zf_posts(creation_date);';
    runQuery($sql);
    $sql = 'create index zfp_modifying_date on zf_posts(modifying_date);';
    runQuery($sql);
}
function parseContent($cont) {
    $cont = str_replace('[blockquote]', '[quote]', $cont);
    $cont = str_replace('[/blockquote]', '[/quote]', $cont);
    $cont = parseUrlTags($cont);
    return $cont;
}
function parseUrlTags($cont) {
    while ((substr($cont,0,5) == '[url=' || strpos($cont, '[url=') != null )&& strpos($cont, '][/url]') != null) {
        $tag_start = strpos($cont, '[url=');
        $tag_end   = strpos($cont, '][/url]') + strlen('][/url]');
        $base_url_starts = strpos($cont, '[url=') + strlen('[url=');
        $base_url_ends = strpos($cont, ' urltitle=');
        $label_starts = strpos($cont, ' urltitle=') + strlen(' urltitle=');
        $label_ends = strpos($cont, '][/url]');
        $base_url = substr($cont, $base_url_starts, $base_url_ends-$base_url_starts);
        $label = substr($cont, $label_starts, $label_ends-$label_starts);
        if (substr($base_url, 0, 7) != 'http://')
            $base_url = "http://".$base_url;
        $new_tag = '[url label="'.$label.'"]'.$base_url.'[/url]';
        $cont = substr($cont, 0, $tag_start).$new_tag.substr($cont, $tag_end);
    }
    return $cont;
}

function splitTopicTable() {
    $sql = 'SELECT * FROM zf_topic WHERE parent_flag = 1 ORDER BY topic_id';
    $result = runQuery($sql);
    $topic_id = 1;
    $post_id = 1;
    while ($line = fetchAssoc($result)) {
        $sql = "INSERT INTO zf_topics (topic_id, forum_id, topic_title, creation_user, creation_date, topic_views, topic_replies, topic_first_post_id, topic_last_post_id, locked_flag, disabled_flag, sticky_flag, poll_flag, system_announcement_flag) VALUES ($topic_id, '".$line['forum_id']."', '".$line['title']."', '".$line['creation_user']."', '".$line['creation_date']."', '".$line['hits']."', '0', '$post_id', '0', '".$line['locked_flag']."', '".$line['disabled_flag']."', '".$line['sticky_flag']."', '".$line['poll_flag']."', '".$line['system_announcement_flag']."')";
        runQuery($sql);
        $sql = "SELECT * FROM zf_topic WHERE parent_id = '".$line['topic_id']."' ORDER BY topic_id";
        $posts = runQuery($sql);
        $first = $post_id;
        $content = parseContent($line['content']);
        $sql = "INSERT INTO zf_posts (post_id, topic_id, content, creation_user, creation_date, modifying_user, modifying_date, ip_address) VALUES ($post_id, $topic_id, '$content', '".$line['creation_user']."', '".$line['creation_date']."', '".$line['modifying_user']."', '".$line['modifying_date']."', '".$line['ip_address']."')";
        runQuery($sql);
        $post_id++;
        $replies = 0;
        while ($post = fetchAssoc($posts)) {
            $content = parseContent($post['content']);
            $sql = "INSERT INTO zf_posts (post_id, topic_id, content, creation_user, creation_date, modifying_user, modifying_date, ip_address) VALUES ($post_id, $topic_id, '$content', '".$post['creation_user']."', '".$post['creation_date']."', '".$post['modifying_user']."', '".$post['modifying_date']."', '".$post['ip_address']."')";
            runQuery($sql);
            $last = $post_id;
            $post_id++;
            $replies++;
        } 
        $sql = "UPDATE zf_topics SET topic_replies = $replies,topic_last_post_id = $last WHERE topic_id = $topic_id;" ;
        runQuery($sql);
        $topic_id++;
    }
    $sql = "select setval('zf_posts_post_id_seq', $post_id);";
    runQuery($sql);
    $sql = "select setval('zf_topics_topic_id_seq', $topic_id);";
    runQuery($sql);
}
function patchForumTable() {
    $sql = 'ALTER TABLE zf_forum ADD forum_topics integer DEFAULT 0 NOT NULL, ADD forum_posts integer DEFAULT 0 NOT NULL, ADD forum_views integer DEFAULT 0 NOT NULL, ADD forum_last_post_id integer DEFAULT 0 NOT NULL, ADD forum_order integer DEFAULT 0 NOT NULL;';
    runQuery($sql);
    $sql = 'SELECT cat_id FROM zf_forum_category;';
    $result = runQuery($sql);
    while ($line = fetchAssoc($result)) {
        $cat_id = $line['cat_id'];
        $sql = "SELECT * FROM zf_forum WHERE cat_id = $cat_id;";
        $forums = runQuery($sql);
        $order = 1;
        while ($forum = fetchAssoc($forums)) {
            $forum_id = $forum['forum_id'];
            $sql = "SELECT sum(topic_views) AS views, sum(topic_replies) AS replies, count(topic_id) AS topics FROM zf_topics WHERE forum_id=$forum_id;";
            $res = runQuery($sql);
            $stats = fetchAssoc($res);
            $sql = "SELECT zp.post_id FROM zf_topics AS zt, zf_posts AS zp WHERE zt.forum_id = $forum_id AND zt.topic_last_post_id = zp.post_id ORDER BY zp.modifying_date desc LIMIT 1";
            $res = runQuery($sql);
            $last = fetchAssoc($res);
            if (! $last) $last['post_id'] = 0;
            if (! $stats['views']) $stats['views'] = 0;
            if (! $stats['replies']) $stats['replies'] = 0;
            $forum_posts = $stats['replies'] + $stats['topics'];
            $sql = "UPDATE zf_forum SET forum_order = $order, forum_last_post_id = ".$last['post_id'].", forum_views = '".$stats['views']."', forum_topics = '".$stats['topics']."', forum_posts = '$forum_posts' WHERE forum_id = $forum_id";
            runQuery($sql);
            $order++;
        }
    }
}
function patchSystemProperies() {
    $sql = "insert into zf_system_properties (property_name, property_desc, property_value) values ('zfsp_preview_topics_limit', 'Last Responses in topic preview mode: Controls the amount of responses (posts) that zForum will show, 0 is unlimited', 5)";
    runQuery($sql); 
    $sql = "insert into zf_system_properties (property_name, property_desc, property_value) values ('zfsp_labeled_links', 'Switch between \"Simple\" and \"Labeled\" links in add post wizard and parser, to activate Labeled links type Yes', '')";
    runQuery($sql);
    $sql = "insert into zf_system_properties (property_name, property_desc, property_value) values ('zfsp_show_ip', 'To show poster IP address to public type Yes', '')";
    runQuery($sql);
    $sql = "insert into zf_system_properties (property_name, property_desc, property_value) values ('zfsp_nested_quotes', 'To allow reply with nested quotes type Yes', 'Yes')";
    runQuery($sql);
    $sql = "insert into zf_system_properties (property_name, property_desc, property_value) values ('zfsp_show_wizards', 'To show wizards used to compose post message type Yes', 'Yes')";
    runQuery($sql);
    $sql = "select setval('zf_system_properties_property_id_seq', 19);";
    runQuery($sql);
}
function deleteTopicTable() {
    $sql = 'ALTER TABLE zf_topic DROP CONSTRAINT zf_topic_forum_id_fkey;';
    runQuery($sql);
    $sql = 'DROP TABLE zf_topic';
    runQuery($sql);
}

?>