-- MySQL Schema for zForum 1.1.0
-- Please report any incompatibilities to julio@techfuel.net
-- Tested on MySQL 4.1.16 (Gentoo Linux and Windows XP Platform)
-- Suggested Database Name: zforum or zforum_new

-- The Following Zope Roles are defined/recognized:
-- Manager - Not a zForum role per se, but it will be recognized also as Administrator
-- zAdministrator - Full Access, can add/edit/remove, user control, policy enforcer
-- zAdministratorStaff - May control certain Categories and the forums within, a moderator, does not have user administration
-- zModerator - Can edit, remove any topic/post on Forum
-- zMemberPremiere - Some Forums may only be available to cetain members (such as paying members) for whatever specific reason
-- zMember - Main, regular membership, can add new postings, edit their own ones, etc
-- Authenticated - Any person logged in the system

-- This is the main categories table, each category may have several sub-categories,
drop table if exists zf_forum_category;
create table zf_forum_category (
   cat_id                       smallint(3) unsigned not null primary key auto_increment,
   cat_name                     varchar(64) not null,
   cat_desc                     varchar(128),
   cat_order                    smallint(3) unsigned not null default 0,
   cat_visible_to               varchar(64) not null
);
insert into zf_forum_category (cat_id, cat_name, cat_desc, cat_order, cat_visible_to) values (1, 'zForum', 'Discussion about zForum, versions, bugs, bigfixes and the like', 1, 'all');
insert into zf_forum_category (cat_id, cat_name, cat_desc, cat_order, cat_visible_to) values (2, 'Public Forums', 'All forums and topics inside this category are viewable/editable by anyone', 2, 'all');
insert into zf_forum_category (cat_id, cat_name, cat_desc, cat_order, cat_visible_to) values (3, 'Member Forums', 'Everyone can view this category, but only registered members can access the topics in its forums', 3, 'all');
insert into zf_forum_category (cat_id, cat_name, cat_desc, cat_order, cat_visible_to) values (4, 'Premium Member Forums', 'Only registered members can view this category and add/edit topics in its forums', 4, 'Authenticated');
insert into zf_forum_category (cat_id, cat_name, cat_desc, cat_order, cat_visible_to) values (5, 'Administration Issues', 'Available to Administrators Only', 5, 'zAdministrator,zAdministratorStaff');

-- zForum Forum Object
-- This is the Topic Container
drop table if exists zf_forum;
create table zf_forum (
   forum_id                    smallint(5) unsigned not null primary key auto_increment,
   cat_id                      smallint(3) unsigned not null,
   forum_title                 varchar(100) not null,
   forum_desc                  varchar(255),
   forum_topics	               int(10) unsigned not null default 0,
   forum_posts                 int(10) unsigned not null default 0,
   forum_views                 int(10) unsigned not null default 0,
   forum_last_post_id          int(10) unsigned not null default 0,
   forum_order                 smallint(5) unsigned not null default 0,
   moderation_flag             tinyint(1) unsigned not null default 0,
   anonymous_viewaccess        tinyint(1) unsigned not null default 1,
   add_postings_access_roles   varchar(64) not null,
   admin_min_access_roles      varchar(64) not null,
   KEY forum_last_post_id(forum_last_post_id),
   index cat_id(cat_id)
);
insert into zf_forum (forum_id, cat_id, forum_title, forum_desc, forum_order, moderation_flag, anonymous_viewaccess, add_postings_access_roles, admin_min_access_roles) values (1, 1, 'zForum 1.0.0 Installation (Includes Beta Versions)', 'Discussions about zForum installation and issues dealing with adding zForum into your zope instance', 1, 0, 1, 'Anonymous,Authenticated', 'zAdministrator,zAdministratorStaff');
insert into zf_forum (forum_id, cat_id, forum_title, forum_desc, forum_order, moderation_flag, anonymous_viewaccess, add_postings_access_roles, admin_min_access_roles) values (2, 1, 'zForum 1.0.0 Usage (Includes Beta Versions)', 'How to use zForum, including the administrative interfaces', 2, 0, 1, 'Anonymous,Authenticated', 'zAdministrator,zAdministratorStaff');
insert into zf_forum (forum_id, cat_id, forum_title, forum_desc, forum_order, moderation_flag, anonymous_viewaccess, add_postings_access_roles, admin_min_access_roles) values (3, 1, 'zForum 1.0.0 Bugs/Issues (Includes Beta Versions)', 'Report any bugs here, although there is a bugtracking program, a lot will be scheduled from here', 3, 0, 1, 'Anonymous,Authenticated', 'zAdministrator,zAdministratorStaff');
insert into zf_forum (forum_id, cat_id, forum_title, forum_desc, forum_order, moderation_flag, anonymous_viewaccess, add_postings_access_roles, admin_min_access_roles) values (4, 1, 'zForum 1.0.0 Tweaks/Wishlists (Includes Beta Versions)', 'Have a suggestion? a better way to do something in zForum?, come inside', 4, 0, 1, 'Anonymous,Authenticated', 'zAdministrator,zAdministratorStaff');
insert into zf_forum (forum_id, cat_id, forum_title, forum_desc, forum_order, moderation_flag, anonymous_viewaccess, add_postings_access_roles, admin_min_access_roles) values (5, 2, 'Test zForum Here', 'The topics inside this forum are deleted regularily, do not post anything important here.', 1, 0, 0, 'Anonymous,Authenticated', 'zAdministrator,zAdministratorStaff');
insert into zf_forum (forum_id, cat_id, forum_title, forum_desc, forum_order, moderation_flag, anonymous_viewaccess, add_postings_access_roles, admin_min_access_roles) values (6, 3, 'zForum bugs and performance reviews based on the testing from our members', 'Write your reviews or comments of your testings in this forum. You need to be registered to add/edit topics', 1, 0, 1, 'Authenticated', 'zAdministrator,zAdministratorStaff');
insert into zf_forum (forum_id, cat_id, forum_title, forum_desc, forum_order, moderation_flag, anonymous_viewaccess, add_postings_access_roles, admin_min_access_roles) values (7, 4, 'zForum Premium Service', 'A special customization is needed? - this is the place', 1, 0, 1, 'Authenticated', 'zAdministrator,zAdministratorStaff');
insert into zf_forum (forum_id, cat_id, forum_title, forum_desc, forum_order, moderation_flag, anonymous_viewaccess, add_postings_access_roles, admin_min_access_roles) values (8, 5, 'Policies on Banning Users', 'Your administrator input is appreciated', 1, 0, 0, 'Authenticated', 'zAdministrator,zAdministratorStaff');

-- Topics in Forums, improvs faster get to first and last post
drop table if exists zf_topics;
create table zf_topics (
   topic_id                    int(10) unsigned not null primary key auto_increment,
   forum_id                    smallint(5) unsigned not null,
   topic_title                 varchar(128) not null,
   topic_desc                  varchar(255),
   creation_user               varchar(40) not null,
   creation_date               datetime not null,
   topic_views                 int(10) unsigned not null default 0,
   topic_replies               int(10) unsigned not null default 0,
   topic_first_post_id         int(10) unsigned not null default 0,
   topic_last_post_id          int(10) unsigned not null default 0,
   locked_flag                 tinyint(1) unsigned not null default 0,
   disabled_flag               tinyint(1) unsigned not null default 0,
   sticky_flag                 tinyint(1) unsigned not null default 0,
   poll_flag                   tinyint(1) unsigned not null default 0,
   system_announcement_flag    tinyint(1) unsigned not null default 0,
   index forum_id(forum_id),
   index system_announcement_flag(system_announcement_flag),
   index topic_creation_date(creation_date)
);
-- Posts in topics
drop table if exists zf_posts;
create table zf_posts (
   post_id		               int(10) unsigned not null primary key auto_increment,
   topic_id                    int(10) unsigned not null,
   content                     text not null,
   creation_user               varchar(40) not null,
   creation_date               datetime not null,
   modifying_user              varchar(40) not null,
   modifying_date              datetime not null,
   ip_address                  varchar(15),
   index topic_id(topic_id),
   index post_creation_date(creation_date),
   index post_modifying_date(modifying_date)
);

-- Poll Support
-- topic_id must be a PARENT topic all the time
-- NOTE This feature is still not available
drop table if exists zf_polls;
create table zf_polls (
  poll_id                     int(8) unsigned not null primary key auto_increment,
  topic_id                    int(8) unsigned not null references zf_topics(topic_id) on delete cascade,
  response                    varchar(128) not null,
  picked                      int(8) unsigned not null default 0,
  index topic_id(topic_id)
);
-- Make sure topic_id 1 (above) has the poll_flag set (1) and the question
-- To be: What Zope version do you currently use more often? (or something in those lines)
-- insert into zf_polls (topic_id, response, picked) values (1, 'Zope 2.6.x', 10);
-- insert into zf_polls (topic_id, response, picked) values (1, 'Zope 2.7.x', 20);
-- insert into zf_polls (topic_id, response, picked) values (1, 'Zope 2.8.x', 16);
-- insert into zf_polls (topic_id, response, picked) values (1, 'Zope X3', 5);

drop table if exists zf_system_properties;
create table zf_system_properties (
  property_id                  smallint(5) unsigned primary key not null auto_increment,
  property_name                varchar(128) not null,
  property_desc                text not null,
  property_value               varchar(128)
);
insert into zf_system_properties (property_name, property_desc, property_value) values ('zfsp_threads_per_page', 'Threads per Page: Number of results per page to show in the topic display section', '20');
insert into zf_system_properties (property_name, property_desc, property_value) values ('zfsp_allow_polls', 'Polls Allowance: Leave empty to disable the ability of adding polls by the users, any other value enable them', 'yes');
insert into zf_system_properties (property_name, property_desc, property_value) values ('zfsp_email_pwd_on_signup', 'Email Password Upon Signup: Regadless of what signup method is used, the new user will receive a welcome email', '');
insert into zf_system_properties (property_name, property_desc, property_value) values ('zfsp_topic_teaser_length', 'Topic Teaser Length: The number of characters shown for a topic when it is expanded from the topic listing page', '250');
insert into zf_system_properties (property_name, property_desc, property_value) values ('zfsp_allow_member_avatars', 'Allow Avatars: If empty, users will not be given the choice of adding or change their avatars, any other value will enable avatars for all registered users in the system', 'yes');
insert into zf_system_properties (property_name, property_desc, property_value) values ('zfsp_system_language', 'System Language: This value will be used when a user accesses the system (Anonymous User), once the user is signed in, he/she will have the opportunity of changing the language settings for her sessions. The language code must match any of the languages defined in the Available Languages section.', 'en_US');
insert into zf_system_properties (property_name, property_desc, property_value) values ('zfsp_admin_contact_email', 'Admin Contact: (Important) - The forum system uses this value to specify the *From*; email header for any email that is sent out, please use a valid email address that your domain will recognize, otherwise your system may not send emails at all', '');
insert into zf_system_properties (property_name, property_desc, property_value) values ('zfsp_use_ranking_system', 'Use Ranking System: If empty, the forum will use default values of Member, Forum Administrator or System Administrator, any other value will enable forum rankings based on the number of postings the users have (see table zf_member_rank for rank information)', 'yes');
insert into zf_system_properties (property_name, property_desc, property_value) values ('zfsp_hot_topic_threshold', 'Hot Topic Threshold: Number of views necessary to mark the topic as *hot*', '50');
insert into zf_system_properties (property_name, property_desc, property_value) values ('zfsp_available_skins', 'Available Skins: Make sure your skins are comma-separated and they all exists in the Style Sheet page', 'zforum');
insert into zf_system_properties (property_name, property_desc, property_value) values ('zfsp_member_quota', 'Message Quota: Leave empty to disable quotas for messages for your users, any other numeric value will represent the number of <b>bytes</b> of allowance, an invalid amount will always default to 50Kb per user', '50000');
insert into zf_system_properties (property_name, property_desc, property_value) values ('zfsp_system_announcement_max', 'System Announcements View: This controls the (maximum) number of system announcements that zForum will display in its right nav, an invalid value or zero will show a =-No System Messages-= title', '5');
insert into zf_system_properties (property_name, property_desc, property_value) values ('zfsp_latest_postings_max', 'Latest Postings View: This controls the (maximum) number of latest postings that zForum will display in its right nav, an invalid value or zero will show a =-No Messages-= title', '5');
insert into zf_system_properties (property_name, property_desc, property_value) values ('zfsp_responses_per_page', 'Responses per page: Controls the amount of responses (children topics) that zForum will show and will add pagination accordingly', '15');
insert into zf_system_properties (property_name, property_desc, property_value) values ('zfsp_header_html', 'Header Text: This can contain html code and will be shown at the top-right corner of zForum', 'Welcome to zForum');
insert into zf_system_properties (property_name, property_desc, property_value) values ('zfsp_available_languages', 'Available languages to the user, use the form: Language:languagecode_COUNTRYCODE, (e.g. US English:en_US). Separate each set by the PIPE Symbol (|)', 'English (U.S.A):en_US|Spanish (M&eacute;xico):es_MX');
insert into zf_system_properties (property_name, property_desc, property_value) values ('zfsp_friendly_url', 'Friendly URL: Add any value to turn on Friendly URLs, google will thank you.', '');
insert into zf_system_properties (property_name, property_desc, property_value) values ('zfsp_allow_preview_topics', 'Allow Topic Preview: Any value in this field will allow preview topics in the page, note this causes a performance hit int he system', 'Yes');
insert into zf_system_properties (property_name, property_desc, property_value) values ('zfsp_preview_topics_limit', 'Last Responses in topic preview mode: Controls the amount of responses (posts) that zForum will show, 0 is unlimited', '5');
insert into zf_system_properties (property_name, property_desc, property_value) values ('zfsp_labeled_links', 'Switch between "Simple" and "Labeled" links in add post wizard and parser, to activate Labeled links type Yes', '');
insert into zf_system_properties (property_name, property_desc, property_value) values ('zfsp_show_ip', 'To show poster IP address to public type Yes', '');
insert into zf_system_properties (property_name, property_desc, property_value) values ('zfsp_nested_quotes', 'To allow reply with nested quotes type Yes', 'Yes');
insert into zf_system_properties (property_name, property_desc, property_value) values ('zfsp_show_wizards', 'To show wizards used to compose post message type Yes', 'Yes');
-- Member Properties
drop table if exists zf_member_properties_skel;
create table zf_member_properties_skel (
  property_id                  smallint(5) unsigned primary key not null auto_increment,
  property_name                varchar(128) not null,
  property_desc                text not null,
  member_editable              tinyint(1) not null default 0
);
-- insert into zf_member_properties_skel (property_name, property_desc) values ('acl_user', 'Member ID', 0);
insert into zf_member_properties_skel (property_id, property_name, property_desc, member_editable) values (1,  'zfmp_username', 'Member Full Name', 1);
insert into zf_member_properties_skel (property_id, property_name, property_desc, member_editable) values (2,  'zfmp_email', 'Member Email', 1);
insert into zf_member_properties_skel (property_id, property_name, property_desc, member_editable) values (3,  'zfmp_last_login', 'Last Login', 0);
insert into zf_member_properties_skel (property_id, property_name, property_desc, member_editable) values (4,  'zfmp_last_login_ip', 'Last Login IP', 0);
insert into zf_member_properties_skel (property_id, property_name, property_desc, member_editable) values (5,  'zfmp_allow_pm_use', 'Allow be Contacted by Other Members', 1);
insert into zf_member_properties_skel (property_id, property_name, property_desc, member_editable) values (6,  'zfmp_postings', 'Number of Postings', 0);
insert into zf_member_properties_skel (property_id, property_name, property_desc, member_editable) values (7,  'zfmp_locale', 'Language for display', 1);
insert into zf_member_properties_skel (property_id, property_name, property_desc, member_editable) values (8,  'zfmp_skin', 'Skin to Use', 1);
insert into zf_member_properties_skel (property_id, property_name, property_desc, member_editable) values (9,  'zfmp_signature', 'Member Signature', 1);
insert into zf_member_properties_skel (property_id, property_name, property_desc, member_editable) values (10, 'zfmp_web_page', 'Member URL', 1);
insert into zf_member_properties_skel (property_id, property_name, property_desc, member_editable) values (11, 'zfmp_country', 'Member Country', 1);
insert into zf_member_properties_skel (property_id, property_name, property_desc, member_editable) values (12, 'zfmp_join_date', 'Join Date', 0);
insert into zf_member_properties_skel (property_id, property_name, property_desc, member_editable) values (13, 'zfmp_banned', 'Controls User Banning Status', 0);

drop table if exists zf_member_properties;
create table zf_member_properties (
  entry_id                     int(10) unsigned primary key not null auto_increment,
  acl_user                     varchar(40) not null,
  property_id                  smallint(5) unsigned not null references zf_member_properties_skel(property_id) on delete cascade,
  property_value               varchar(128),
  index property_id(property_id),
  index acl_user(acl_user),
  index property_value(property_value),
  index user_property(acl_user, property_id)
);

-- The Avatar container had to be changed from binary (typea or blob)
-- to "text" which will contain the image encoded in base64
drop table if exists zf_member_avatars;
create table zf_member_avatars (
  avatar_id                    int(10) unsigned primary key not null auto_increment,
  content_type                 varchar(128) not null,
  acl_user                     varchar(40) not null,
  avatar_image text
);

-- Member Subscriptions
drop table if exists zf_member_subscriptions;
CREATE TABLE zf_member_subscriptions (
  entry_id                     int(10) unsigned not null primary key auto_increment,
  acl_user                     varchar(40) not null references zf_member_properties(acl_user) on delete cascade,
  subscription_id              int(10) unsigned,
  subscription_type            enum('f', 't'),
  INDEX acl_user(acl_user),
  INDEX subscription_id(subscription_id),
  INDEX user_subscription(acl_user,subscription_id)
);

drop table if exists zf_pm_categories;
create table zf_pm_categories (
  cat_id                       smallint(5) unsigned not null primary key auto_increment,
  display_order                smallint(5) unsigned not null default 0,
  name                         varchar(50)
);
insert into zf_pm_categories (name, display_order) values ('Inbox', 1);
insert into zf_pm_categories (name, display_order) values ('Read', 2);
insert into zf_pm_categories (name, display_order) values ('Sent', 3);
insert into zf_pm_categories (name, display_order) values ('Trash', 4);

drop table if exists zf_pm;
create table zf_pm (
  pm_id                        int(10) unsigned not null primary key auto_increment,
  cat_id                       smallint(5) unsigned not null references zf_pm_categories(cat_id) on delete cascade,
  read_flag                    tinyint(1) unsigned not null default 0,
  acl_user                     varchar(128) not null,
  from_acl_user                varchar(128) not null,
  subject                      varchar(50) not null,
  message                      text not null,
  creation_date                datetime not null,
  index cat_id(cat_id),
  index acl_user(acl_user),
  index read_flag(read_flag)
);
insert into zf_pm (cat_id, acl_user, read_flag, from_acl_user, subject, message, creation_date) values (1, 'member', 0, 'administrator', 'Welcome to zForum', 'Just wanted you to welcome to the forums', NOW());
insert into zf_pm (cat_id, acl_user, read_flag, from_acl_user, subject, message, creation_date) values (1, 'member', 0, 'member', 'Email Sent to Self', 'You can also do this to send messages to yourself', NOW());
insert into zf_pm (cat_id, acl_user, read_flag, from_acl_user, subject, message, creation_date) values (1, 'member', 1, 'someuser', 'This is an example of a read message', 'This is just a simple\ntest message with read status', NOW());
insert into zf_pm (cat_id, acl_user, read_flag, from_acl_user, subject, message, creation_date) values (2, 'member', 1, 'anotheruser', 'Read messages should be here', 'Use this folder to store your read messages.', NOW());

-- Available Languages table
drop table if exists zf_available_languages;
create table zf_available_languages (
  language_code                varchar(5) primary key not null,
  language_desc                varchar(40)
);
insert into zf_available_languages (language_code, language_desc) values ('en_us', 'English (USA)');
insert into zf_available_languages (language_code, language_desc) values ('es_mx', 'EspaÃÂ±ol (MÃÂ©xico)');

-- User Ranges, Number of posts (or less) for the user and will be considered:
-- Guess where the ranking system comes from? :)
drop table if exists zf_member_rank;
create table zf_member_rank (
  rank_id                      smallint(5) unsigned primary key not null auto_increment,
  rank_name                    varchar(128) not null,
  rank_value_min               mediumint(8) not null,
  index rank_value_min(rank_value_min)
);
insert into zf_member_rank (rank_value_min, rank_name) values (0, 'Starfleet Ensign');
insert into zf_member_rank (rank_value_min, rank_name) values (6, 'Starfleet Lieutenant, Junior Grade');
insert into zf_member_rank (rank_value_min, rank_name) values (15, 'Starfleet Lieutenant');
insert into zf_member_rank (rank_value_min, rank_name) values (40, 'Starfleet Lieutenant Commander');
insert into zf_member_rank (rank_value_min, rank_name) values (85, 'Starfleet Commander');
insert into zf_member_rank (rank_value_min, rank_name) values (110, 'Starfleet Captain');
insert into zf_member_rank (rank_value_min, rank_name) values (160, 'Starfleet Commodore');
insert into zf_member_rank (rank_value_min, rank_name) values (200, 'Starfleet Rear Admiral');
insert into zf_member_rank (rank_value_min, rank_name) values (250, 'Starfleet Vice Admiral');
insert into zf_member_rank (rank_value_min, rank_name) values (500, 'Starfleet Admiral');
insert into zf_member_rank (rank_value_min, rank_name) values (1000, 'Starfleet Fleet Admiral');

