# zForum 1.0 Zope Product
# http://www.zforum.org/
#
# Project Started: May 12, 2005
# Zope version tested on: Zope 2.8.5
# Python version tested on: Python 2.3.5
# MySQL version tested on: 4.0.24 / 4.1.16
# PostgreSQL version tested on: 7.4.11 / 8.0.1
#
# Creator: Julio Flores Schwarzbeck (speedbird)
# Co-creator: Marcin F. Wisniowski
#
# http://www.zope.org/Members/speedbird/
# http://www.julioflores.com/ (Spanish)
# http://www.codeit.com/
# Developed at: Las Vegas, NV, USA
# This product is dedicated to Diego, Pablo and Ines.
#
# zForum is an Open Source Message Board Product written
# for the Z Object Publishing Environment (Zope2)
# 2005 Julio Flores Schwarzbeck, zforum@zforum.org
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the ZPL (Zope Public License).
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# ZPL Zope Public License (ZPL.txt) for more details.

__doc__     = 'zForum - Zope/SQL Based Message Board Product'
__version__ = '1.1.0'
__author__  = 'Julio Flores Schwarzbeck'

# Zope "Magic" including security machinery
from AccessControl import ClassSecurityInfo
from AccessControl.PermissionRole import rolesForPermissionOn
from Globals import DTMLFile
from Globals import InitializeClass
from Globals import Persistent
from OFS.Folder import Folder
from OFS import ObjectManager

# Miscellaneous necessary imports to be used, such as email capabilities,
from Products.MailHost import MailHost
# Importing Some Safe Python Scripts Into ZODB
from Products.PythonScripts.PythonScript import manage_addPythonScript, PythonScript
# Processing of ZPT/DTML, 
from Products.PageTemplates.ZopePageTemplate import manage_addPageTemplate
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from webdav.WriteLockInterface import WriteLockInterface
# Get the location(s) of the possible Products
from App.FindHomes import ppath as PRODUCTS_PATH
# Some light-weight XML parsing functions
from xml.sax.saxutils import escape
from xml.sax.saxutils import unescape

# Image Conversion from a BLOB into an image object to grab properties
from cStringIO import StringIO
# New for 1.0.3, Images (Avatars will be base64-encoded to assure compatibility among DB systems
import base64
import struct

# RSS Support: PyRSS2Gen 0.1.1
# Thanks to Aurhor Andrew Dalke <dalke@dalkescientific.com> for this sweet contribution to python
from PyRSS2Gen import PyRSS2Gen
import datetime

# External Dependancies:

# For form-based authentication, look for CookieCrumbler in CMF
# otherwise try to locate it as a standalone product.
cookie_crumbler_error = 0
try:
  from Products.CookieCrumbler import CookieCrumbler
except ImportError:
  cookie_crumbler_error = 1
  
if cookie_crumbler_error:
  # Look CookieCrumbler in Plone, otherwise error out
  # The next line apparently causes problems with zope
  # instances that have CMF/Plone but not a standalone
  # instance of CookieCrumbler, so the next line:
  ###from Products.CMFCore.CookieCrumbler import CookieCrumbler
  # Becomes
  from Products.CMFCore import CookieCrumbler
  # Because the way CK is located in CMF, the "CookieCrumbler" will contain the namespace for the manage_addCC
  # Please someone verify this?

# Misc imports to make development life easier, also part of the zope's "magic" (zope2)
import os
import string
import re
import random
import Acquisition
import AccessControl
import math
from DateTime import DateTime

# Some modules to allow parsing of message contents,
# don't know if I need all of them at this stage, though
import sgmllib

# For Reverse DNS functions such as for getting the domain name from an IP
import socket

# DB-"Independant" SQL Methods
from Products.ZSQLMethods import SQL

# Constructor interface UI for the forum instance
manage_addZForumForm = PageTemplateFile('manage_dir/add_zforum_form.pt', globals())

# add_zforum_form will post here:
def manage_addZForum(self, id, REQUEST=None):
  """Adds a new zForum Instance"""
  self._setObject(id, zForum(id, REQUEST))
  # Default Permissions Settings
  changePermissions(self, id)
  # Set Forum Roles, Plone Aware
  setForumRoles(self, id, REQUEST.get('plone_instance', ''))
  return self.manage_main(self, REQUEST)

# Main Class: Folderish (persistent)
class zForum(Folder,
             Acquisition.Implicit,
             AccessControl.Role.RoleManager,
             Persistent,
             ObjectManager.ObjectManager,):
  """ zForum - SQL Based Zope Message Board System """
  meta_type  = 'zForum'
  
  # Instantiate Security Machinery
  security = ClassSecurityInfo()

  # Function:zfGenId():Generates a Pseudo-random Id
  def zfGenId():
    return str(random.random())[2:]

  global db_conn_id
  db_conn_id = 'zf_database_connection'

  # And the "stamping inside zodb"
  def __init__(self, id, REQUEST):
    """Initialize new zForum"""
    self.id = id
    self.title = REQUEST.get('title', '')
    if REQUEST is not None:
      self.zfSetupSystem(REQUEST)

  def zfSetupSystem(self, REQUEST):
    # Basically here add what needs to be added to 
    # start operating the forums, such as creation
    # of the roles in ZODB, addition of MailHost
    # and CookieCrumbler, and any other ZODB/SQL
    # Additions needed
    plone_instance = REQUEST.get('plone_instance', '')
    # Create Roles:
    #for role in ['zAdministrator', 'zAdministratorStaff', 'zModerator', 'zMember', 'zMemberPremiere']:
    #  self._addRole(role=role)
    
    # Add acl_users instance if REQUESTed
    if REQUEST.get('add_user_instance', '') != '' and not plone_instance:
      self.manage_addUserFolder()
      # Create two users on this newly created instance
      user_instance = getattr(self, 'acl_users')
      user_instance._addUser('administrator', 'administrator', 'administrator', ['zAdministrator'], '')
      user_instance._addUser('moderator', 'moderator', 'moderator', ['zModerator'], '')
      user_instance._addUser('member', 'member', 'member', ['zMember'], '')

      # Add CookieCrumbler Instance (if User Addition was also selected)
      CookieCrumbler.manage_addCC(self, 'cookie_authentication')

    # Add Database Connection
    db_system_type = REQUEST.get('db_system_type', 'mysql')
    if db_system_type == 'mysql':
      #DA.manage_addZMySQLConnection(self, REQUEST.get('db_conn_id', 'database_connection'), '', REQUEST.get('db_conn_str', 'zforum@localhost zforum zforum'))    
      from Products.ZMySQLDA import DA    
      DA.manage_addZMySQLConnection(self, db_conn_id, '', REQUEST.get('db_conn_str', 'zforum@localhost zforum zforum'))
    elif db_system_type == 'postgres':
      from Products.ZPsycopgDA import DA
      DA.manage_addZPsycopgConnection(self, db_conn_id, '', REQUEST.get('db_conn_str', 'host=localhost user=zforum dbname=zforum password=zforum'), zdatetime='YES')

    # MailHost Settings, only if mail_host is checked, SMTP defaults to 25 always, changable later
    if REQUEST.get('mail_host', ''):
      MailHost.manage_addMailHost(self, 'MailHost', '', REQUEST.get('mail_host'))
    else:
      MailHost.manage_addMailHost(self, 'MailHost')


    # Here's the deal:
    #
    # All presentation (Page Templates and CSS) go in ZODB
    # All Scripts go here (zope-protected)

    # Step I - Create the necessary support folders under the main instance
    # --------------------------------------------------------------
    folder_list = ['zAdmin', 'zImages', 'zSQL', 'zUI', 'zJavaScript']
    for folder in folder_list:
      self.manage_addFolder(id=folder, title='')
    # Emoticons Container
    self.zImages.manage_addFolder(id='emoticons', title='')
    # Toolbar Container
    self.zImages.manage_addFolder(id='toolbar', title='')

    # Step II - Addition of page templates/dtml (CSS & JS Code), and ZSQL Methods
    # ---------------------------------------------------------------------------
    for k in PRODUCTS_PATH:
      location = os.path.join(k, 'zForum', 'instance_dir')
      if os.path.exists(location):
        files_dir = location
        break
    for file in os.listdir(files_dir):
      if file.rfind('.pt') >= 0 or file.rfind('.dtml') >= 0 or file.rfind('.ps') >= 0:
        f_han  = open(os.path.join(files_dir, file))
        f_data = f_han.read()
        f_han.close()
      if file.rfind('.pt') >= 0:
        if plone_instance:
          f_data = f_data.replace('<doc metal:use-macro="here/zf_master_template/macros/zf_doctype" />', '')
          f_data = f_data.replace('here/zf_master_template/macros/zf_master_template', 'here/main_template/macros/master')
          if file == 'zf_macros.pt':
            f_data = f_data.replace('user_prefs_form', 'user_prefs_plone_form')
            f_data = f_data.replace('here/zf_macros/macros/auth_links', 'here/zf_macros/macros/auth_links_plone')
            f_data = f_data.replace('here/zf_macros/macros/noauth_links', 'here/zf_macros/macros/noauth_links_plone')
        manage_addPageTemplate(self, id=file[:file.rfind('.pt')], title='', text=f_data)
      elif file.rfind('.dtml') >= 0:
        self.addDTMLMethod(id=file[:file.rfind('.dtml')], title='', file=f_data)
      elif file.rfind('.ps') >= 0:
        manage_addPythonScript(self, id=file[:file.rfind('.ps')])
        self._getOb(file[:file.rfind('.ps')]).write(f_data)
        
    for k in PRODUCTS_PATH:
      location = os.path.join(k, 'zForum', 'instance_dir', 'zAdmin')
      if os.path.exists(location):
        files_dir = location
        break
    folder_ob = self._getOb('zAdmin')
    for file in os.listdir(files_dir):
      f_han  = open(os.path.join(files_dir, file))
      f_data = f_han.read()
      f_han.close()
      if file.rfind('.pt') >= 0:
        if plone_instance:
          f_data = f_data.replace('<doc metal:use-macro="here/zf_master_template/macros/zf_doctype" />', '')
          f_data = f_data.replace('here/zf_master_template/macros/zf_master_template', 'here/main_template/macros/master')
          if file == 'zf_macros.pt':
            f_data = f_data.replace('user_prefs_form', 'user_prefs_plone_form')
            f_data = f_data.replace('here/zf_macros/macros/auth_links', 'here/zf_macros/macros/auth_links_plone')
            f_data = f_data.replace('here/zf_macros/macros/noauth_links', 'here/zf_macros/macros/noauth_links_plone')
        manage_addPageTemplate(folder_ob, id=file[:file.rfind('.pt')], title='', text=f_data)
      elif file.rfind('.dtml') >= 0:
        folder_ob.addDTMLMethod(id=file[:file.rfind('.dtml')], title='', file=f_data)
      elif file.rfind('.ps') >= 0:
        manage_addPythonScript(folder_ob, id=file[:file.rfind('.ps')])
        folder_ob._getOb(file[:file.rfind('.ps')]).write(f_data)

    # Addition of CSS 2.0
    for k in PRODUCTS_PATH:
      location = os.path.join(k, 'zForum', 'instance_dir', 'zUI')
      if os.path.exists(location):
        files_dir = location
        break
    folder_ob = self._getOb('zUI')
    for file in os.listdir(files_dir):
      f_han  = open(os.path.join(files_dir, file))
      f_data = f_han.read()
      f_han.close()
      if file.rfind('.pt') >= 0:
        manage_addPageTemplate(folder_ob, id=file[:file.rfind('.pt')], title='', text=f_data)
      elif file.rfind('.dtml') >= 0:
        folder_ob.addDTMLMethod(id=file[:file.rfind('.dtml')], title='', file=f_data)
        
    # Addition of JavaScript
    for k in PRODUCTS_PATH:
      location = os.path.join(k, 'zForum', 'instance_dir', 'zJavaScript')
      if os.path.exists(location):
        files_dir = location
        break
    folder_ob = self._getOb('zJavaScript')
    for file in os.listdir(files_dir):
      f_han  = open(os.path.join(files_dir, file))
      f_data = f_han.read()
      f_han.close()
      if file.rfind('.dtml') >= 0:
        folder_ob.addDTMLMethod(id=file[:file.rfind('.dtml')], title='', file=f_data)

    # Images
    for k in PRODUCTS_PATH:
      location = os.path.join(k, 'zForum', 'instance_dir', 'zImages')
      if os.path.exists(location):
        files_dir = location
        break
    folder_ob = self.zImages
    for file in os.listdir(files_dir):
      if file != 'emoticons' and file != 'toolbar':
        f_han  = open(os.path.join(files_dir, file), "rb")
        f_data = f_han.read()
        f_han.close()
        folder_ob.manage_addImage(file, f_data)
    # Images - Emoticons
    for k in PRODUCTS_PATH:
      location = os.path.join(k, 'zForum', 'instance_dir', 'zImages', 'emoticons')
      if os.path.exists(location):
        files_dir = location
        break
    folder_ob = self.zImages.emoticons
    for file in os.listdir(files_dir):
      f_han  = open(os.path.join(files_dir, file), "rb")
      f_data = f_han.read()
      f_han.close()
      fname = file[:-4]
      folder_ob.manage_addImage(file, f_data, ':%s:' % fname)
    # Images - Toolbar
    for k in PRODUCTS_PATH:
      location = os.path.join(k, 'zForum', 'instance_dir', 'zImages', 'toolbar')
      if os.path.exists(location):
        files_dir = location
        break
    folder_ob = self.zImages.toolbar
    for file in os.listdir(files_dir):
      f_han  = open(os.path.join(files_dir, file), "rb")
      f_data = f_han.read()
      f_han.close()
      fname = file[:-4]
      folder_ob.manage_addImage(file, f_data)
    # ZSQL Public Methods
    for k in PRODUCTS_PATH:
      location = os.path.join(k, 'zForum', 'instance_dir', 'zSQL')
      if os.path.exists(location):
        files_dir = location
        break
    folder_ob = self._getOb('zSQL')
    for file in os.listdir(files_dir):
      f_han  = open(os.path.join(files_dir, file))
      f_data = f_han.read()
      f_han.close()
      # Some parsing for some DB-Specific System
      if db_system_type == 'mysql':
        loc_start = f_data.find('[POSTGRES]')
        loc_end   = f_data.find('[/POSTGRES]')
        if loc_start >= 0:
          f_data = '%s%s' % (f_data[:loc_start], f_data[loc_end + 11:])
        f_data = f_data.replace('[MYSQL]', '')
        f_data = f_data.replace('[/MYSQL]', '')
      else:
        loc_start = f_data.find('[MYSQL]')
        loc_end   = f_data.find('[/MYSQL]')
        if loc_start >= 0:
          f_data = '%s%s' % (f_data[:loc_start], f_data[loc_end + 8:])
        f_data = f_data.replace('[POSTGRES]', '')
        f_data = f_data.replace('[/POSTGRES]', '')
      obj_id = file[:file.rfind('.zsql')]
      if file.rfind('.zsql') >= 0:
        # Some Parsing Needed to successfully "create" a valid
        # ZSQL Method - Thanks Kapil Thangavelu
        parameters={}
        start = f_data.find('<dtml-comment>')
        end   = f_data.find('</dtml-comment>')
        if start == -1 or end == -1 or start > end:
          raise ValueError,'ZSQL Method Creation - Could not find parameter block on %s' % file
        block = f_data[start + 14:end]
        for line in block.split('\n'):
          pair = line.split(':', 1)
          if len(pair) != 2:
            continue
          parameters[pair[0].strip().lower()]=pair[1].strip()
            
        arguments = parameters.get('arguments', '').replace(',', '\n')
        
        SQL.manage_addZSQLMethod(folder_ob,
                                 id=obj_id,
                                 title='',
                                 connection_id=db_conn_id,
                                 arguments=arguments,
                                 template=f_data)
        # Change some properties in the newly created sql method
        getattr(self.zSQL, file[:file.rfind('.zsql')]).manage_advanced(0,   # max_rows
                                                                      100, # max_cache
                                                                      0,   # cache_time
                                                                      '',  # don't really see any point in allowing
                                                                      '')  # brain specification...
  # Custom Permissions Declaration
  security.setPermissionDefault('Admin zForums', ('Manager', 'zAdministrator', 'zAdministratorStaff',))
  security.setPermissionDefault('Moderator zForums', ('zModerator',))
  security.setPermissionDefault('Access zForum Personal Messages', ('Manager', 'zAdministrator', 'zModerator', 'zAdministratorStaff', 'zMember', 'zMemberPermiere',))

  # zForum Methods
  # ---------------------------------------------------------------------------
  
  # Function:zfChangeAnonLanguage:Changes the default system language for anonymous users
  security.declarePublic('zfChangeAnonLanguage')
  def zfChangeAnonLanguage(self):
    """ Changes the default system language for anonymous users """
    req  = self.REQUEST
    dest = req.get('chglang_dest', '')
    qs   = req.get('chglang_qs', '')
    new_lang = req.get('new_lang', '')

    if new_lang:
      req.RESPONSE.setCookie('zforum_language', new_lang, path='/')

    if dest:
      if qs:
        return req.RESPONSE.redirect('%s?%s' % (dest, qs))
      else:
        return req.RESPONSE.redirect(dest)
    else:
      return req.RESPONSE.redirect('%s/index_html' % (self.zfForumBase()))

  # Function:zfChangeLanguage():Changes the Authenticated User's Language Settings
  security.declarePublic('zfChangeLanguage')
  def zfChangeLanguage(self):
    """ Changes the Authenticated User's Language Settings """
    req = self.REQUEST
    user = req['AUTHENTICATED_USER']
    default_lang = self.zfGetUserProperty('zfmp_locale', user.getUserName(), 'en_US')
    req.RESPONSE.setCookie('zforum_language', default_lang, path='/')
    pass
    

  # Function:zfConvertAvatars():Converts avatars for pre-1.0.3 version of zForum
  # in the new, base64 encoded format
  security.declareProtected('Admin zForums', 'zfConvertAvatars')
  def zfConvertAvatars(self):
    """ Converts avatars for pre-1.0.3 version of zForum - Callable from Web - For Admin Only """
    # This assumes that you have done the following previous to the conversion:
    # - alter table zf_member_avatars add column new_avatar_image text;
    avatars = self.zSQL.getAllAvatars()
    for avatar in avatars:
      self.zSQL.saveNewAvatar(avatar_id=avatar.avatar_id, new_avatar_image=base64.encodestring(avatar.avatar_image))
    # Once this is done, you need to do the following..
    # - alter table zf_member_avatars change column avatar_image old_avatar_image blob;
    # - alter table zf_member_avatars change column new_avatar_image avatar_image text;
    return "Converted.."
  
  # Function:zfPaginationWidget():Returns the "root" of the zForum system
  security.declarePublic('zfPaginationWidget')
  def zfPaginationWidget(self, total, start, href, ptype='forum'):
    """ A Pagination Widget """

    # total = total number of results 
    # start = zero-based index of first result to be displayed (e.g. page 3 of 5 10-batch pages starts with 20)
    # href  = url + necessary querystring for your results

    # Items per page
    if ptype == 'forum':
      batch = int(self.zfGetSystemProperty('zfsp_threads_per_page', 15))
    elif ptype == 'topic':
      batch = int(self.zfGetSystemProperty('zfsp_responses_per_page', 15))
    else:
      batch = 15
    
    # current page number. add first to prevent zero division errors.
    current = (start+batch)/batch

    # total number of pages. use ceiling to catch a remainder.
    pages = int(math.ceil(float(total)/float(batch)))

    # Need to modify the widget to return a more user-friendly pagination UI, in the form:
    # [Previous 10] [Previous] 1 2 3 4 5 6 7 [8] 9 10 [Next] [Next 10]
    # If Click on "next 10" then show:
    # [Previous 10] [Previous] 11 12 13 14 15 16 17 [18] 19 20 [Next] [Next 10]
    # Assuming there are 25 "pages":
    # [Previous 10] [Previous] 21 22 23 24 [25] [Next] [Next 10]
    # Things to consider:
    # - If on page 28 of a 32 page widget, click on "Next 10" must go to the "last" (32) number, not 38
    # - "Previous", "Previous 10", "Next" and "Next 10" must be disabled when appropriate

    PAGE_SET = 10
    sets_to_display = pages / PAGE_SET
    reminder_pages  = pages % PAGE_SET
    if reminder_pages:
      extra_page = 1
    else:
      extra_page = 0
    sets_to_display += extra_page

    real_sets_to_display = total / float(batch)

    if real_sets_to_display > 1:
      html_code = '<div class="breadcrumbs" style="text-align:center;"> %s:\n' % (self.tr('global_pages', 'Pages'))
      
      if start >= (batch * PAGE_SET):
        html_code += '[<a class="breadcrumbs" href="%s&amp;start=%s">Previous %s</a>]&nbsp;' % (href, start - (batch * PAGE_SET), PAGE_SET)

      if start - batch >= 0:
        html_code += '[<a class="breadcrumbs" href="%s&amp;start=%s" title="">Previous</a>]&nbsp;' % (href, start - batch)
     
      # Need to find the right "range", for example on a 10-page set of 3-results-per-page for 44 total results,
      # If I click page 6, the range must be 1 2 3 4 5 6 7 8 9 10
      # If I am on page 11, then click on 15, the range must be 11 12 13 14 15 16 17 18 19 20
      # and so on (Note 16 17 18 19 20 must be disabled)
      start_number = 0
      end_number = PAGE_SET
      mid_number = (start / batch) + 1
      while 1:
        if mid_number >= start_number and mid_number <= end_number:
          break
        start_number = end_number
        end_number = start_number + PAGE_SET

      page = start_number * batch
        
      this_range = range(1 + start_number, end_number + 1)

      for idx in this_range:
        if idx == (start + batch) / batch:
          html_code += '<b>%s</b>&nbsp;' % str(idx)
        else:
          if idx * batch - batch < total:
            html_code += '<a class="breadcrumbs" href="%s&amp;start=%s" title="">%s</a>&nbsp;' % (href, page, idx)
          
        page += batch

      if start + batch < total:
        html_code += '[<a class="breadcrumbs" href="%s&amp;start=%s" title="">Next</a>]&nbsp;' % (href, start + batch)
     
      if start + (batch * PAGE_SET) < total:
        tot = start + (batch * PAGE_SET) 
        html_code += '[<a class="breadcrumbs" href="%s&amp;start=%s" title="">Next %s</a>]&nbsp;' % (href, tot, PAGE_SET)

      html_code += '</div>\n'
    else:
      html_code = ''
        
    return html_code

  # Function:zfCountParentTopicsInForum(forum_id):Returns the number of parent topics in a forum
  security.declarePublic('zfCountParentTopicsInForum')
  def zfCountParentTopicsInForum(self, forum_id):
    """ Returns the number of parent topics in a forum """
    return self.zSQL.countParentTopicsInForum(forum_id=forum_id)[0].topic_count

  # Function:zfCountChildrenTopicsInTopic(topic_id):Returns the number of children topics for a parent
  security.declarePublic('zfCountPostsInTopic')
  def zfCountPostsInTopic(self, topic_id):
    """ Returns the number of parent topics in a forum """
    return self.zSQL.countPostsInTopic(topic_id=topic_id)[0].posts_count
    
  # Function:zfForumBase():Returns the "root" of the zForum system
  security.declarePublic('zfForumBase')
  def zfForumBase(self):
    """ Returns the forum root folder """
    return self.absolute_url()


  # Function:zfGetCategoriesAndForums():Grabs all the categories and the forums available for them, and returns a nifty list containing dictionaries of categories/forums
  security.declarePublic('zfGetCategoriesAndForums')
  def zfGetCategoriesAndForums(self):
    user       = self.REQUEST['AUTHENTICATED_USER']
    today      = DateTime()
    cat_list   = []
    cat_dict   = {}
    cats_and_forums = self.zSQL.getCategoriesAndForums()
    if cats_and_forums:
      for data in cats_and_forums:
        cat_id   = data.cat_id
        cat_name = data.cat_name
        cat_desc = data.cat_desc
        # This var controls if the category (and its forums)
        # will be available to the user
        cat_view_access = 0
        cat_visible_to = data.cat_visible_to.split(',')
        if 'all' in cat_visible_to or user.has_permission('Admin zForums', self):
          cat_view_access = 1
        else:
          if [cat for cat in cat_visible_to if user.has_role(cat)]:
            cat_view_access = 1
        if cat_view_access:
          forum_id = data.forum_id
          if forum_id:
            forum_title           = data.forum_title
            forum_desc            = data.forum_desc
            moderation_flag       = data.moderation_flag
            topics                = int(data.forum_topics)         
            posts                 = int(data.forum_posts)
            views                 = int(data.forum_views)
            last_updated          = data.modifying_date
            last_updated_by       = data.modifying_user
            last_updated_topic_id = data.topic_id

            # Now, lets' set a flag to mark the "last updated" message in
            # red (or any other css) if the message is less than a day old (24 hs)
            if last_updated:
              if (today - last_updated) <= 1.0:
                update_flag = 1
              else:
                update_flag = ''
            else:
              update_flag = ''

            anonymous_viewaccess      = data.anonymous_viewaccess
            add_postings_access_roles = data.add_postings_access_roles
            admin_min_access_roles    = data.admin_min_access_roles
            # Now here is the deal, not everyone will be able to view/enter
            # a particular forum/category, so here is when the logic applies
            # We'll check if the user logged in or not has the role needed
            # note that a forum could have the role "Anonymous", which
            # would give an anonymous user access to the forum's topics
            add_forum = 0
            for role in add_postings_access_roles.split(','):
              if user.has_role(role) or anonymous_viewaccess:
                add_forum = 1
                break
            if add_forum:
              forum_dict = {'forum_id': forum_id,
                            'forum_title': self.zfParse(forum_title),
                            'forum_desc': forum_desc,
                            'anonymous_viewaccess': anonymous_viewaccess,
                            'topics': topics,
                            'posts': posts,
                            'views': views,
                            'last_updated': last_updated,
                            'last_updated_by': last_updated_by,
                            'last_updated_topic_id': last_updated_topic_id,
                            'update_flag': update_flag}
              if cat_dict.has_key(cat_id):
                cat_dict['forum_list'].append(forum_dict)
              else:
                cat_dict = {cat_id: cat_id,
                            'cat_id': cat_id,
                            'cat_name': cat_name,
                            'forum_list': [forum_dict],
                            'cat_visible_to': data.cat_visible_to,
                            'cat_desc': cat_desc}
                cat_list.append(cat_dict)
          else:
            # Cat Visible but No Forums
            cat_dict = {cat_id: cat_id,
                        'cat_id': cat_id,
                        'cat_name': cat_name,
                        'forum_list': [],
                        'cat_visible_to': data.cat_visible_to,
                        'cat_desc': cat_desc}
            cat_list.append(cat_dict)
    return cat_list

  # Function:zfGetForumsStats():Provides some totals and intresting numbers for the Information Page
  security.declarePublic('zfGetForumsStats')
  def zfGetForumsStats(self):
    some_stats = self.zSQL.getForumsStats()
    if some_stats:
      if some_stats[0].topics:
        topics = some_stats[0].topics
      else:
        topics = 0
      if some_stats[0].posts:
        posts = some_stats[0].posts
      else:
        posts = 0
      if some_stats[0].views:
        views = some_stats[0].views
      else:
        views = 0
    else:
      topics  = 0
      posts   = 0
      views   = 0
    ret_dict = {'topics' : int(topics),
                'posts'  : int(posts),
                'views'  : int(views)}
    return ret_dict
  
  security.declarePublic('zfCountTopics')
  def zfCountTopics(self):
    topic_count = self.zSQL.countTopics()[0].topics
    if not topic_count:
      topic_count = 0    
    return int(topic_count)  
  # zfGetRecentPosts
  # Returns the latest posts from the selected category (or any category)
  # Relies on the XXX system property 
  security.declarePublic('zfGetRecentPosts')
  def zfGetRecentPosts(self, cat_id=0):
    return []

  # Function:zfGetVersion:Returns the zForum Version taken directly from zope
  security.declarePublic('zfGetVersion')
  def zfGetVersion(self):
    """ Returns the Forum Version, Web Callable """
    return self.getPhysicalRoot().Control_Panel.Products.zForum.version

  # Function:zfGetZopeInternals():Grabs some info about the Zope's control panel
  security.declareProtected('Admin zForums', 'zfGetZopeInternals')
  def zfGetZopeInternals(self):
    cp = self.getPhysicalRoot().Control_Panel
    try:
      users = len(self.acl_users.getUserNames())
    except:
      users = 'n/a'
    if hasattr(self, db_conn_id):
      db = getattr(self, db_conn_id)
      db_system = db.meta_type
    else:
      db_system = 'Unknown'
    rval = {'zope_version'  : cp.version_txt(),
            'python_version': cp.sys_version(),
            'platform'      : cp.sys_platform(),
            'uptime'        : cp.process_time(),
            'db_system'     : db_system,
            'users'         : users}
    return rval

  # Function:zfGetSystemAnnouncements():Returns a "quick" dict with topic_id and topic_title keys to be displayed in zForum's right nav bar
  security.declarePublic('zfGetSystemAnnouncements')
  def zfGetSystemAnnouncements(self):
    try:
      max_announcements = int(self.zfGetSystemProperty('zfsp_system_announcement_max', '0'))
    except ValueError:
      max_announcements = 0
    if max_announcements:
      system_announcements = self.zSQL.getSystemAnnouncements(max_announcements=max_announcements)
    else:
      system_announcements = []
    return system_announcements

  # zfGetLatestTopics
  # Returns a "quick" dict with topic_id and topic_title keys
  # to be displayed in zForum's right nav bar
  security.declarePublic('zfGetLatestTopics')
  def zfGetLatestTopics(self):
    try:
      max_announcements = int(self.zfGetSystemProperty('zfsp_latest_postings_max', '0'))
    except ValueError:
      max_announcements = 0
    if max_announcements:
      latest_topics = self.zSQL.getLatestTopics(max_announcements=max_announcements)
    else:
      latest_topics = []
    return latest_topics
  # Method: zfGetLastPostAnchor( ) : Returns link to last post in topic
  security.declarePublic('zfPostAnchor')
  def zfLastPostAnchor(self, topic_id):
    topic          = self.zSQL.getTopic(topic_id=topic_id)[0]
    if not topic:
      return self.zfForumBase()
    posts_per_page = int(self.zfGetSystemProperty('zfsp_responses_per_page', 15))
    frendly_urls   = self.zfGetSystemProperty('zfsp_friendly_url', '')    
    posts_in_topic = topic.topic_replies + 1
    post_id        = topic.topic_last_post_id
    start          = int(posts_in_topic/posts_per_page) * posts_per_page
    
    if frendly_urls:
      url_anchor = '%s/view_topic/%s/%s#%s' % (self.zfForumBase(), topic_id, start, post_id)
    else:
      url_anchor = '%s/view_topic?topic_id=%s&start=%s#%s' % (self.zfForumBase(), topic_id, start, post_id)
    return url_anchor   
  # Function:zfGetSystemProperty(property, default_value):Returns a system's property value if found, otherwise it returns the default value passed
  # Handles the properties stored system-wide,
  # returns information such as the number of postings to show in the preview pane, etc
  security.declarePublic('zfGetSystemProperty')
  def zfGetSystemProperty(self, property, default):
    sql_system_prop = self.zSQL.getSystemProperty(property=property)
    if sql_system_prop and sql_system_prop[0].property_value:
      system_prop = sql_system_prop[0].property_value
    else:
      system_prop = default
    return system_prop

  # Function:zfGetSystemProperties():Returns a list of all system properties (Admin Purposes)
  security.declareProtected('Admin zForums', 'zfGetSystemProperties')
  def zfGetSystemProperties(self):
    all_properties = self.zSQL.getSystemProperties()
    # This *has* to be true all the time, otherwise most likely you have a
    # big big problem in your hands..
    if all_properties:
      return all_properties
    else:
      return {}

  # Function:zfPutSystemProperty(property, new_value):Updates an existing system property
  # Handles changing/addition of a system property
  security.declareProtected('Admin zForums', 'zfPutSystemProperty')
  def zfPutSystemProperty(self, property, new_value):
    self.zSQL.putSystemProperty(property=property, new_value=new_value)
    pass
  
  # Function:zfGetUserProperty(property,username,default):returns a property for a user
  # Handles the properties stored per user basis,
  # returns information such as the number of postings for the user, etc
  security.declarePublic('zfGetUserProperty')
  def zfGetUserProperty(self, property, username, default):
    #### I've decided to remove this next code below and make this method
    #### not web-callable to be able to grab any user's properties
    # Basically Admins can view any member's properties, and
    # regular users can only view their own properties
    #req  = self.REQUEST
    #user = req['AUTHENTICATED_USER']
    allowed = 1
    #if self.zfIsAdmin():
    #  allowed = 1
    #elif user.getUserName() == username and user.has_role('Authenticated'):
    #  allowed = 1

    if allowed:
      user_props = self.zSQL.getUserProperty(user=username, property=property)
      if user_props and user_props[0].acl_user:
        user_property = user_props[0].property_value
        if not user_property:
          # Prop exist, but it can be an empty string, NULL, None, etc
          user_property = default
      else:
        user_property = default
    else:
      user_property = default
    return user_property

  # zfPutUserProperty
  # Adds/Changes a user property, such as the last login time for the user, etc
  security.declarePublic('zfPutUserProperty')
  def zfPutUserProperty(self, user, property, new_value):
    # Careful with this one, change prop only of the current user
    req = self.REQUEST
    auth_user    = req['AUTHENTICATED_USER']
    h_property   = property
    if user == auth_user.getUserName() or auth_user.has_permission('Admin zForums', self):
      # Maybe this needs some checking..
      property_id = self.zSQL.getPropertyFromSkel(property=property)[0].property_id
      # Check if an actual record for this property_id exists, if it does, update, otherwise create the prop
      property_exists = self.zSQL.getUserProperty(user=user, property=property)
      if property_exists and property_exists[0].property_id:
        self.zSQL.updateUserProperty(user=user, property_id=property_id, new_value=new_value)
      else:
        self.zSQL.putUserProperty(user=user, property_id=property_id, new_value=new_value)
        
    return h_property

  # zfPutUserPropertyRestricted
  # Very similar to zfPutUserProperty but overrides authentication,
  # And only works for certain properties (as in a new signup, when
  # the user is still not authenticated
  security.declarePublic('zfPutUserPropertyRestricted')
  def zfPutUserPropertyRestricted(self, user, property, new_value):
    auth_props = ['zfmp_username', 'zfmp_email', 'zfmp_join_date']
    if property in auth_props:
      property_id = self.zSQL.getPropertyFromSkel(property=property)[0].property_id
      # Check if an actual record for this property_id exists, if it does, update, otherwise create the prop
      property_exists = self.zSQL.getUserProperty(user=user, property=property)
      if property_exists and property_exists[0].property_id:
        self.zSQL.updateUserProperty(user=user, property_id=property_id, new_value=new_value)
      else:
        self.zSQL.putUserProperty(user=user, property_id=property_id, new_value=new_value)

  # zfLogin
  # This function handles the login page
  security.declarePublic('zfLogin')
  def zfLogin(self):
    """ Handles the Main Login Method """
    req       = self.REQUEST
    user      = req['AUTHENTICATED_USER']
    username  = user.getUserName()
    forum_id  = req.get('forum_id', '')
    topic_id  = req.get('topic_id', '')
    dest      = req.get('destination', '')
    came_from = req.get('came_from', '')    

    if user.has_role('Authenticated'):
      # If authenticated, store the login IP and Date/Time
      self.zfPutUserProperty(username, 'zfmp_last_login', DateTime().strftime('%c'))
      ip_addr = req.get('HTTP_X_FORWARDED_FOR', '') # Using Apache/Other Webserver/Proxy? (Florent Carlier)
      if ip_addr == '':
        ip_addr = req.get('REMOTE_ADDR', 'Unknown IP')
      self.zfPutUserProperty(username, 'zfmp_last_login_ip', ip_addr)

      # Set te language
      self.zfChangeLanguage()

      if dest:
        return req.RESPONSE.redirect('%s?forum_id=%s&topic_id=%s' % (dest, forum_id, topic_id))
      elif came_from:
        return req.RESPONSE.redirect(came_from)
      else:
        return req.RESPONSE.redirect('%s/index_html' % (self.zfForumBase()))
    else:
      # Wrong Password or member is not authenticated
      return req.RESPONSE.redirect('%s/login_form?destination=%s&retry=1&forum_id=%s&topic_id=%s' % (self.zfForumBase(), dest, forum_id, topic_id))

  # zfLogout
  # Handles User Logout
  security.declarePublic('zfLogout')
  def zfLogout(self):
    """ Handles the Main Logout Method """
    req      = self.REQUEST
    username = req['AUTHENTICATED_USER'].getUserName()
    req.RESPONSE.expireCookie('__ac', path='/')
    req.RESPONSE.redirect('%s/index_html?user=%s' % (self.zfForumBase(), username))


  # zfForumViews
  # Updates the hits for the forum  
  security.declarePublic('zfUpdateForumViews')  
  def zfUpdateForumViews(self, forum_id):
    self.zSQL.updateForumViews(forum_id=forum_id)
    pass
    
  # zfTopicViews
  # Updates the hits for the topic
  security.declarePublic('zfUpdateHits')
  def zfUpdateTopicViews(self, topic_id):
    self.zSQL.updateTopicViews(topic_id=topic_id)
    pass
    
  # Function:tr(lang_id, word):Translation implementation, this is still experimental, as I still need to find a way to do it elegantly and without having to depend on dozens of "other" products
  security.declarePublic('tr')
  def tr(self, lang_id, word):
    """ Translator Module"""
    # This is a pseudo-translation module, I decided to implement it as propietary to keep
    # zforum from depending on external products, I know some puritans will bash me for
    # not using PTS, or even Localizer, but for the same toke, this way, zForum continues
    # to be self-sustentable
    DEFAULT_LOCALE = 'en_US' # This will be used if all the other options fail, plese do not change it if you don't know how all this works
    req  = self.REQUEST
    user = req['AUTHENTICATED_USER']
    if user.has_role(['Authenticated']):
      if req.get('zforum_language', '') == '':
        # No Language yet set, grab it
        locale = self.zfGetUserProperty('zfmp_locale', user.getUserName(), '')
        if locale == '':
          # No Locale Settings in the profile, grab the default system or default to en_US if all else fails
          locale = self.zfGetSystemProperty('zfsp_system_language', DEFAULT_LOCALE)
        req.RESPONSE.setCookie('zforum_language', locale, path='/')
    else:
      # If there is already a cookie associated with the language, use, otherwise set it..
      if req.get('zforum_language', '') == '':
        locale = self.zfGetSystemProperty('zfsp_system_language', DEFAULT_LOCALE)
        req.RESPONSE.setCookie('zforum_language', locale, path='/')

    
    zforum_language = req.get('zforum_language', DEFAULT_LOCALE)

    # Grab the language files (Note that if you have turned debugging on, this process might be considerably slower)
    for k in PRODUCTS_PATH:
      location = os.path.join(k, 'zForum', 'languages')
      if os.path.exists(location):
        lang_loc = location
        break
    lang_dict = {}
    debug_list = []
    for lang_file in os.listdir(lang_loc):
      if lang_file.find('.zlang') >= 0:
        f_han  = open(os.path.join(lang_loc, lang_file))
        f_data = f_han.readlines()
        lang_definition = 1
        f_han.close()        
        for tmp_data in f_data:
          data = tmp_data.replace('\n', '')
          data = data.replace('\r', '')
          if lang_definition:
            data_key = data
            lang_dict.update({data_key: {}})
            lang_definition = 0
          else:
            if data and data[0] != '#':
              key_value = data.split('::')
              if len(key_value) == 2:
                key   = key_value[0].strip()
                value = key_value[1].strip()
                lang_dict[data_key].update({key:value})

    if lang_dict.has_key(zforum_language):
      translated = lang_dict[zforum_language].get(lang_id, word)
    else:
      translated  = word

    if lang_id == 'trdebug' and word == 'trdebug':
      translated = lang_dict #['es_MX'].get('global_emoticonhelp', 'TEST')
    
    return translated

  # zfGetForum
  # Just a simple wrapper that grabs the forum contents (not its topics)
  security.declarePublic('zfGetForum')
  def zfGetForum(self, forum_id):
    # Security Implementation API to call a protected sql
    forum = self.zSQL.getForum(forum_id=forum_id)
    if forum:
      forum_data = forum[0]
    else:
      forum_data = {}
    return forum_data

  # zfGetForumTopics
  # This is a rather important function that returns all the information
  # needed in the "view_forum" (display of all topics (siblings) page)
  # Note that here mighr be a good place to add some security checkings
  # to prevent unauthorized view of a forum.., in addition, we can
  # eventually pass some parameters such as the sort order
  security.declarePublic('zfGetForumTopics')
  def zfGetForumTopics(self, forum_id, offset):
    # New definition since zForum 1.1.0
    # Topics per page
    limit = int(self.zfGetSystemProperty('zfsp_threads_per_page', 15))
    # Allow Preview Topics
    preview_topics = self.zfGetSystemProperty('zfsp_allow_preview_topics', '')
    # Preview Limit
    preview_limit = self.zfGetSystemProperty('zfsp_preview_topics_limit', '')
    # Teaser Length
    try:
      teaser_len = int(self.zfGetSystemProperty('zfsp_topic_teaser_length', '10'))
    except ValueError:
      teaser_len = 0
      
    today = DateTime()
    topic_list = []
    # Get sticky topics
    sticky_topics = self.zSQL.getForumTopics(forum_id=forum_id, sticky_flag=1, limit=limit, offset=offset)
    count_sticky_topics = len(sticky_topics)
    # Get other topics
    new_limit = limit - count_sticky_topics
    if new_limit > 0:
      normal_topics = self.zSQL.getForumTopics(forum_id=forum_id, sticky_flag=0, limit=new_limit, offset=offset)
    # In the returning list, the sticky topics go first, then the parents
     
    if sticky_topics:
      for topic in sticky_topics:
        if preview_topics:
          posts_preview = self.zSQL.getTopicPreview(topic_id=topic.topic_id, limit=preview_limit)
        else:
          posts_preview = []
        if (today - topic.modifying_date) <= 1.0:
          update_flag = 1
        else:
          update_flag = 0
        topic_list.append({'topic_id':        topic.topic_id,
                           'forum_id':        topic.forum_id,
                           'title':           topic.topic_title,
                           'description':     topic.topic_desc,
                           'author':          topic.creation_user,
                           'published':       topic.creation_date,
                           'views':           topic.topic_views,
                           'replies':         topic.topic_replies,
                           'sticky':          topic.sticky_flag,
                           'locked':          topic.locked_flag,
                           'disabled':        topic.disabled_flag,
                           'sys_anno':        topic.system_announcement_flag,
                           'teaser':          topic.content, 
                           'teaser_len':      teaser_len,
                           'preview':         posts_preview,
                           'last_updated':    topic.modifying_date,
                           'last_updated_by': topic.modifying_user,
                           'update_flag':     update_flag})
    if normal_topics:
      for topic in normal_topics:
        if preview_topics:
          posts_preview = self.zSQL.getTopicPreview(topic_id=topic.topic_id, limit=preview_limit)
        else:
          posts_preview = []
        if (today - topic.modifying_date) <= 1.0:
          update_flag = 1
        else:
          update_flag = 0
        topic_list.append({'topic_id':        topic.topic_id,
                           'forum_id':        topic.forum_id,
                           'title':           topic.topic_title,
                           'description':     topic.topic_desc,
                           'author':          topic.creation_user,
                           'published':       topic.creation_date,
                           'views':           topic.topic_views,
                           'replies':         topic.topic_replies,
                           'sticky':          topic.sticky_flag,
                           'locked':          topic.locked_flag,
                           'disabled':        topic.disabled_flag,
                           'sys_anno':        topic.system_announcement_flag,
                           'teaser':          topic.content, 
                           'teaser_len':      teaser_len,
                           'preview':         posts_preview,
                           'last_updated':    topic.modifying_date,
                           'last_updated_by': topic.modifying_user,
                           'update_flag':     update_flag})
    return topic_list
    

  # Function:zfGetAnyTopic(topic_id):Returns a single topic (Mostly for editing purposes)
  security.declarePublic('zfGetAnyTopic')
  def zfGetAnyTopic(self, topic_id):
    db_topic = self.zSQL.getTopic(topic_id=topic_id)
    if db_topic:
      res_topic = db_topic[0]
    else:
      # This fixes a bug in zope IMHO where zopes page teplates' test() function
      # evaluates the "false" statement even if the condition has been tested
      # to be "true"
      res_topic = {'topic_title': 'N/A'}
    return res_topic
    
  # Function:zfGetAnyPost(topic_id):Returns a single post (Mostly for editing purposes)
  security.declarePublic('zfGetAnyPost')
  def zfGetAnyPost(self, post_id):
    db_post = self.zSQL.getPost(post_id=post_id)
    if db_post:
      res_post = db_post[0]
    else:
      # This fixes a bug in zope IMHO where zopes page teplates' test() function
      # evaluates the "false" statement even if the condition has been tested
      # to be "true"
      res_post = {'topic_id': 0}
    return res_post
    
  # Function:zfGetTopic(topic_id):Grabs a topic and siblings, this generates a single dictionary with the same keys as the zfGetForumTopics method
  security.declarePublic('zfGetTopic')
  def zfGetTopic(self, topic_id, offset):
    # Note, thi will generate a IndexError exception if you are calling a non-existant topic_id
    # Items per page
    limit = int(self.zfGetSystemProperty('zfsp_responses_per_page', 10))
    topic = self.zSQL.getTopic(topic_id=topic_id)[0]
    forum = self.zSQL.getForum(forum_id=topic.forum_id)[0]    
    posts = self.zSQL.getTopicPosts(topic_id=topic_id, limit=limit, offset=offset)
    if not posts:
      posts = []
    topic_dict = {'topic': topic,
                  'forum': forum,
                  'posts': posts}
    return topic_dict
    
  # Function:zfParse(content,emoticons=1,add_breaks=1,preview=None):High-Level HTML sanitizer
  # Parses most of the messages, handles pseudo-html tags, handles emoticons
  # and sanitizes most of the content passed,
  # NOTE that this function depends on the following:
  # - fixTableBreaks()
  security.declarePublic('zfParse')  
  def zfParse(self, content='', emoticons=1, add_breaks=1, preview=None, labels=None): 
    # HTML tags quotation
    tmp_content = self.zfQuoteHTML(content) 
    # URL Tags
    if labels:
      return self.zfStripBBTags(tmp_content)
    if tmp_content.find('[url') >= 0 and tmp_content.find('[/url]') >= 0:
      tmp_content = self.zfParseUrlTags(tmp_content)
    # Add breaks
    if int(add_breaks):
      tmp_content = tmp_content.replace('\n', '<br />')
    # Emoticons parsing
    if emoticons:
      tmp_content = self.zfParseEmoticons(tmp_content)
    # Justify - parse justify befor preview flag check
    tmp_content = tmp_content.replace('[left]', '<div style="text-align: left;">')
    tmp_content = tmp_content.replace('[/left]', '</div>')
    tmp_content = tmp_content.replace('[center]', '<div style="text-align: center;">')
    tmp_content = tmp_content.replace('[/center]', '</div>')
    tmp_content = tmp_content.replace('[right]', '<div style="text-align: right;">')
    tmp_content = tmp_content.replace('[/right]', '</div>')
    tmp_content = tmp_content.replace('[justify]', '<div style="text-align: justify;">')
    tmp_content = tmp_content.replace('[/justify]', '</div>')
    # Preview
    if preview:
      return self.zfStripBBTags(tmp_content)
    # Quote Tags
    if tmp_content.find('[quote') >= 0 and tmp_content.find('[/quote]') >= 0:
      tmp_content = self.zfParseQuoteTags(tmp_content)
    
    # Miscellaneous html tags (presentation)
    tmp_content = tmp_content.replace('[b]','<b>')
    tmp_content = tmp_content.replace('[/b]','</b>')
    tmp_content = tmp_content.replace('[i]','<i>')
    tmp_content = tmp_content.replace('[/i]','</i>')
    tmp_content = tmp_content.replace('[big]','<span style="font-size:large;">')
    tmp_content = tmp_content.replace('[/big]','</span>')
    tmp_content = tmp_content.replace('[small]','<span style="font-size:10px;">')
    tmp_content = tmp_content.replace('[/small]','</span>')
    tmp_content = tmp_content.replace('[img]', '<img src="')
    tmp_content = tmp_content.replace('[/img]', '" style="border:1px solid black;" />')
    tmp_content = tmp_content.replace('[ol]', '<ol>')
    tmp_content = tmp_content.replace('[ul]', '<ul>')
    tmp_content = tmp_content.replace('[/ol]', '</ol>')
    tmp_content = tmp_content.replace('[/ul]', '</ul>')
    tmp_content = tmp_content.replace('[li]', '<li>')
    tmp_content = tmp_content.replace('[/li]', '</li>')
    #Hidden Tags
    rnd_id = str(random.random())[2:]
    tmp_content = tmp_content.replace('[hidden]', '<a href="#" onclick="toggle_thread(\'id_%s\');return false">%s</a>\n<div class="standout" id="id_%s" style="display:none;">' % (rnd_id, self.tr('global_messageedited', 'Message has been edited, Click Here to view/hide original message'), rnd_id))
    tmp_content = tmp_content.replace('[/hidden]', '</div>')
  
    # Table Support (Very Basic): Probably Need CSS definitions (table Information)
    tmp_content = tmp_content.replace('[table]', '<table class="ftable" cellspacing="0" cellpadding="0">')
    tmp_content = tmp_content.replace('[/table]', '</table>')
    tmp_content = tmp_content.replace('[th]', '<th class="cell3d">')
    tmp_content = tmp_content.replace('[/th]', '</th>')
    tmp_content = tmp_content.replace('[td]', '<td class="cell3d">')
    tmp_content = tmp_content.replace('[/td]', '</td>')
    tmp_content = tmp_content.replace('[tr]', '<tr>')
    tmp_content = tmp_content.replace('[tr0]', '<tr class="shadelight">')
    tmp_content = tmp_content.replace('[tr1]', '<tr class="shadedark">')
    tmp_content = tmp_content.replace('[tr2]', '<tr class="shadedarker">')
    tmp_content = tmp_content.replace('[/tr]', '</tr>')
    tmp_content = tmp_content.replace('[/tr0]', '</tr>')
    tmp_content = tmp_content.replace('[/tr1]', '</tr>')
    tmp_content = tmp_content.replace('[/tr2]', '</tr>')
    # Unsupported/Nonstandard MS-style
    tmp_content = tmp_content.replace('[blink]','<blink>')
    tmp_content = tmp_content.replace('[/blink]','</blink>')
    tmp_content = tmp_content.replace('[u]','<u>')
    tmp_content = tmp_content.replace('[/u]','</u>')
    tmp_content = tmp_content.replace('[marquee]','<marquee>')
    tmp_content = tmp_content.replace('[/marquee]','</marquee>')
    # Code tags, I am not converting straight [code] to <code>
    # since I need the paragraph break that <pre> gives, suit this
    # to your needs
    tmp_content = tmp_content.replace('[code]', '<div class="code">')
    tmp_content = tmp_content.replace('[/code]', '</div>')
    # Signature Handling
    tmp_content = tmp_content.replace('[signature]', '<pre class="pre">')
    tmp_content = tmp_content.replace('[/signature]', '</pre>')

    # There seems to be a problem with tables and parsing, since the system will insert a <br /> even between table elements, like
    # <br /><table cellspacing="0" cellpadding="0">
    # <br /><tr><td>Data</td></tr>
    # <br /><tr><td>Data</td></tr>
    # <br /></table>
    # This is invalid in the XHTML specification, and besides, it creates
    # unnecesary breaks BEFORE the table, so I'll replace back the (invalid) <br />
    # I find between te table definition and continue with the process.
    # Update zForum 1.0.5+ - This also applies to <pre> and <code> tags..
    if tmp_content.find('<table') >= 0 or tmp_content.find('<pre') >= 0:
      tmp_content = self.fixTagBreaks('<table', '</table>', tmp_content)
      tmp_content = self.fixTagBreaks('<pre', '</pre>', tmp_content)
      tmp_content = self.fixTagBreaks('<code', '</code>', tmp_content)
    return tmp_content
  
  # HTML Quotatin method takes care of '<', '>', '$', ...
  security.declarePublic('zfQuoteHTML') 
  def zfQuoteHTML(self, content):
    return escape(content)
  # HTML Reverse Quote method  
  security.declarePublic('zfUnQuoteHTML') 
  def zfUnQuoteHTML(self, content):
    return unescape(content)
       
  # Function:zfParseHiddenTags(content):Removes hidden pseudo-html tags. This is a stub function for use with zfParse()
  security.declarePublic('zfParseHiddenTags')
  def zfParseHiddenTags(self, topic_content):
    """ removes the [hidden][/hidden] pseudo-html tag attributes """
    start_tag = topic_content.find('[hidden]')
    end_tag   = topic_content.rfind('[/hidden]')
    parsed_content = topic_content
    if start_tag >= 0 and end_tag >= 0:
      parsed_content = '%s%s' % (topic_content[:start_tag], topic_content[end_tag + 9:])
    return parsed_content
    
  # Function:zfParseURLTags(content):Converts url tags into a href HTML sequence. This is a stub function for use with zfParse()  
  security.declarePrivate('zfParseUrlTags')  
  def zfParseUrlTags(self, content):
    labeled_links = self.zfGetSystemProperty('zfsp_labeled_links', '')
    while content.find('[url') >= 0 and content.find('[/url]') >= 0:
      tag_start = content.find('[url')
      tag_end   = content.find('[/url]') + len('[/url]')
      if content.find('[url]') >= 0:
        #Simple URL TAG:  [url]http://www.foo.com[/url]
        base_url_start = content.find('[url]') + len('[url]')
        base_url_end   = content.find('[/url]', base_url_start - 1)
        base_url       = content[base_url_start:base_url_end]
        if base_url[0:7] != 'http://':
          base_url = 'http://' + base_url
        new_tag = '<a onclick="window.open(this.href);return false;" title="" href="' + base_url + '">' + base_url + '</a>'
      if content.find('[url label="') >= 0:
        # Labeled URL TAG: [url label="foo link"]http://www.foo.com[/url]
        label_start    = content.find('[url label="') + len('[url label="')
        label_end      = content.find('"]', label_start)
        label          = content[label_start:label_end]
        base_url_start = content.find('[url label="' + label + '"]') + len('[url label="' + label + '"]')
        base_url_end   = content.find('[/url]', base_url_start - 1)
        base_url       = content[base_url_start:base_url_end]
        if base_url[0:7] != 'http://':
          base_url = 'http://' + base_url
        if labeled_links:
          new_tag = '<a onclick="window.open(this.href);return false;" title="" href="' + base_url + '">' + label + '</a>'
        else:
          new_tag = '<a onclick="window.open(this.href);return false;" title="" href="' + base_url + '">' + base_url + '</a>'
      content = '%s%s%s' % (content[:tag_start], new_tag, content[tag_end:])
    return content

  # Function:zfParseQuoteTags(content):Converts url tags into a href HTML sequence. This is a stub function for use with zfParse()  
  security.declarePrivate('zfParseQuoteTags')  
  def zfParseQuoteTags(self, content):
    if content.find('[quote]') >= 0:
      # simple Qoute format: [quote]Content[/quote]
      content = content.replace('[quote]', '<div class="quote">')
    if content.find('[quote from="') >= 0:
      # extended Qoute format with option from: [quote from="username"]Content[/quote]
      content = content.replace('[quote from="', '<div class="quote_title"> ' + self.tr('global_from_user', 'From') + ': ')
      content = content.replace('"]', '</div><div class="quote">')
    content = content.replace('[/quote]', '</div>')
    return content

  security.declarePrivate('zfParseEmoticons')
  def zfParseEmoticons(self, content):
    content = content.replace(':angry:', '<img alt="Angry" src="%s/emoticons/angry.gif" style="height:15px;width:15px;border:0px;" />' % self.zImages.absolute_url())
    content = content.replace(':cheesy:', '<img alt="Cheesy" src="%s/emoticons/cheesy.gif" style="height:15px;width:15px;border:0px;" />' % self.zImages.absolute_url())
    content = content.replace(':cool:', '<img alt="Cool!" src="%s/emoticons/cool.gif" style="height:15px;width:15px;border:0px;" />' % self.zImages.absolute_url())  
    content = content.replace(':cry:', '<img alt="Cry" src="%s/emoticons/cry.gif" style="height:15px;width:15px;border:0px;" />' % self.zImages.absolute_url())
    content = content.replace(':embarassed:', '<img alt="Embarrassed" src="%s/emoticons/embarassed.gif" style="height:15px;width:15px;border:0px;" />' % self.zImages.absolute_url())
    content = content.replace(':grin:', '<img alt="Frin" src="%s/emoticons/grin.gif" style="height:15px;width:15px;border:0px;" />' % self.zImages.absolute_url())
    content = content.replace(':huh:', '<img alt="Huh?" src="%s/emoticons/huh.gif" style="height:22px;width:15px;border:0px;" />' % self.zImages.absolute_url())
    content = content.replace(':kiss:', '<img alt="Kiss" src="%s/emoticons/kiss.gif" style="height:15px;width:15px;border:0px;" />' % self.zImages.absolute_url())  
    content = content.replace(':laugh:', '<img alt="Laugh" src="%s/emoticons/laugh.gif" style="height:15px;width:15px;border:0px;" />' % self.zImages.absolute_url())
    content = content.replace(':lol:', '<img alt="LOL" src="%s/emoticons/laugh.gif" style="height:15px;width:15px;border:0px;" />' % self.zImages.absolute_url())
    content = content.replace(':lipsrsealed:', '<img alt="Lips Sealed" src="%s/emoticons/lipsrsealed.gif" style="height:15px;width:15px;border:0px;" />' % self.zImages.absolute_url())
    content = content.replace(':rolleyes:', '<img alt="Roll Eyes" src="%s/emoticons/rolleyes.gif" style="height:15px;width:15px;border:0px;" />' % self.zImages.absolute_url())
    content = content.replace(':sad:', '<img alt="Sad" src="%s/emoticons/sad.gif" style="height:15px;width:15px;border:0px;" />' % self.zImages.absolute_url())
    content = content.replace(':shocked:', '<img alt="Shocked" src="%s/emoticons/shocked.gif" style="height:15px;width:15px;border:0px;" />' % self.zImages.absolute_url())
    content = content.replace(':smiley:', '<img alt="Smile!" src="%s/emoticons/smiley.gif" style="height:15px;width:15px;border:0px;" />' % self.zImages.absolute_url())
    content = content.replace(':smile:', '<img alt="Smile!" src="%s/emoticons/smiley.gif" style="height:15px;width:15px;border:0px;" />' % self.zImages.absolute_url())
    content = content.replace(':tongue:', '<img alt="Tongue" src="%s/emoticons/tongue.gif" style="height:15px;width:15px;border:0px;" />' % self.zImages.absolute_url())
    content = content.replace(':undecided:', '<img alt="Undecided" src="%s/emoticons/undecided.gif" style="height:15px;width:15px;border:0px;" />' % self.zImages.absolute_url())
    content = content.replace(':wink:', '<img alt="Wink" src="%s/emoticons/wink.gif" style="height:15px;width:15px;border:0px;" />' % self.zImages.absolute_url())  
    content = content.replace(':~(', '<img alt="Cry" src="%s/emoticons/cry.gif" style="height:15px;width:15px;border:0px;" />' % self.zImages.absolute_url()) 
    content = content.replace(':(', '<img alt="Sad" src="%s/emoticons/sad.gif" style="height:15px;width:15px;border:0px;" />' % self.zImages.absolute_url()) 
    content = content.replace(':|', '<img alt="Shocked" src="%s/emoticons/shocked.gif" style="height:15px;width:15px;border:0px;" />' % self.zImages.absolute_url())  
    content = content.replace(':)', '<img alt="Smile" src="%s/emoticons/smiley.gif" style="height:15px;width:15px;border:0px;" />' % self.zImages.absolute_url())    
    content = content.replace(';)', '<img alt="Wink" src="%s/emoticons/wink.gif" style="height:15px;width:15px;border:0px;" />' % self.zImages.absolute_url())
    content = content.replace(':D', '<img alt="LOL" src="%s/emoticons/laugh.gif" style="height:15px;width:15px;border:0px;" />' % self.zImages.absolute_url())
    content = content.replace(':p', '<img alt="Tongue" src="%s/emoticons/tongue.gif" style="height:15px;width:15px;border:0px;" />' % self.zImages.absolute_url())
    return content
  
  security.declarePrivate('zfStripBBTags')
  def zfStripBBTags(self, content):
    # Regular Expressions of BB tags
    tags = "b|i|u|big|small|img|ol|ul|li|table|tr[012]?|th|td|blink|marquee|code|quote(''| from=\"(.*)\")|hidden\](.*)\[/hidden"
    exp = '\[/?('+tags+')\]'
    parser = re.compile(exp)
    return parser.sub('', content)
     
  # Function:zfQuoteFormat(raw_text, creation_user):Handles quoting somebody else's postings
  security.declarePublic('zfQuoteFormat')
  def zfQuoteFormat(self, raw_text, creation_user):
    nested_quotes = self.zfGetSystemProperty('zfsp_nested_quotes', '')
    quote_content = self.zfUnQuoteHTML(raw_text)
    if not nested_quotes:
      #Remove any previouse quotes, leave only last posted message 
      if quote_content.find('[quote') >= 0:
        quote_start = quote_content.find('[quote')
        if quote_content.rfind('[/quote]') >= 0:
          quote_end  = quote_content.rfind('[/quote]') + len('[/quote]')
          quote_content = "%s%s" % (quote_content[:quote_start], quote_content[quote_end:])
    quote_content = quote_content.replace("\r", "\\r")
    quote_content = quote_content.replace("\n", "\\n")
    quote_content = quote_content.replace("'", "\\'")
    # Truncate "singnatures", i.e. do not quote signatures if the message comes with one
    if quote_content.find('[signature]') >= 0:
      quote_content = quote_content[:qoute_content.find('[signature]')]
    return "[quote from=\"%s\"]%s[/quote]" % (creation_user, quote_content)
  # removes <br /> tag from HTML bolck tags like <table> or <pre>, still experimental
    
  security.declarePrivate('fixTagBreaks')
  def fixTagBreaks(self, start_tag, end_tag, content):
    """ removes the <br /> from [table][/table], [pre][/pre] and [code][/code] pseudo-html tag """
    tag_starts = content.find(start_tag)
    tag_ends   = content.find(end_tag)
    if tag_starts >= 0 and tag_ends >= 0:
      before_tag   = content[:tag_starts]
      tag_content  = content[tag_starts:tag_ends]
      after_tag    = content[tag_ends:]
      content      = '%s%s%s' % (before_tag, tag_content.replace('<br />', '\n'), after_tag)
    return content
    
  # Function:zfGenericId:Pseudo-random ID generator
  # For generating pseudo-random id's from the UI
  security.declarePublic('zfGenericId')
  def zfGenericId(self):
    return str(random.random())[2:]


  # Function:zfGetForumList():Returns an available forum_id/forum_title dictionary based on the user
  # Simple API to return the forumid/forumtitle
  # Mainly intended for the use on a drop-down style
  # combo box.
  # Make sure we apply some security here, eventually, for example,
  # do not return unauthorized forums
  security.declarePublic('zfGetForumList')
  def zfGetForumList(self):
    req  = self.REQUEST
    user = req['AUTHENTICATED_USER']
    all_forums = self.zSQL.getForums()
    forum_list = []
    for forum in all_forums:
      add_forum = 0
      for role in forum.add_postings_access_roles.split(','):
        if user.has_role(role):
          add_forum = 1
          break
      if add_forum:
        forum_list.append({'forum_id': forum.forum_id,
                           'forum_title': forum.forum_title})
    return forum_list

  # Function:zfUpdateSubscribers(forum_id,topic_id,content) - Search all members subscribed to a specific topic or the entire forum, and send them an email
  security.declarePublic('zfUpdateSubscribers')
  def zfUpdateSubscribers(self, forum_id, topic_id, content):
    """ Email Subscribed users when a forum/topic is updated """
    friendly_urls = self.zfGetSystemProperty('zfsp_friendly_url', '')
    if friendly_urls:
      url = '%s/view_topic/%s/0' % (self.zfForumBase(), topic_id)
    else:
      url = '%s/view_topic?topic_id=%s' % (self.zfForumBase(), topic_id)
    # This may become a little bit expensive
    forum = self.zSQL.getForum(forum_id=forum_id)
    topic = self.zSQL.getTopic(topic_id=topic_id)
    mail_code = 0
    subscribed_users = self.zSQL.getSubscribers(forum_id=forum_id, topic_id=topic_id)
    # Note: subscribed_users is already "deduped" (grouped by acl_user)
    # Outgoing messages shluld also be "translated", let's see what the community suggests
    if subscribed_users:
      mfrom = self.zfGetSystemProperty('zfsp_admin_contact_email', '')
      subject = '%s: %s' % (self.tr('zforum_update', 'zForum Update'), forum[0].forum_title)
      body = """A thread in zForum for which you are subscribed has been added or replied to:

Topic Title: %s

Latest Response: %s

Topic Link: %s

If you wish to stop receiving these updates, please log in via %s/index_html, edit your member properties and remove your current forum/topic subscriptions. Thank You.

zForum Administrator.""" % (topic[0].topic_title, content, url, self.absolute_url())
      for subscribed_user in subscribed_users:
        mto = self.zfGetUserProperty('zfmp_email', subscribed_user.acl_user, '')
        if mto != '' and mfrom != '':
          mail_code = self.zfSimpleSendMessage(mto, mfrom, subject, body)
        else:
          mail_code = -1
    return mail_code

# Function zfAddTopic(Multiple Parameters):Handles the addition of a topic on a specific forum
  security.declarePublic('zfAddTopic')
  def zfAddTopic(self):
    """ Handles the addition of a topic on a specific forum """
    req      = self.REQUEST
    forum_id = req['forum_id'] # Must exist, I won't even do any checking here..
    user     = req['AUTHENTICATED_USER']
    is_auth  = user.has_role('Authenticated')
    can_post = self.zfCanPost(forum_id)

    friendly_urls = self.zfGetSystemProperty('zfsp_friendly_url', '')
    
    preview  = req.get('preview', '')
    if preview:
      return self.add_topic_form(self, REQUEST=req)
    else:
      # Ready for adding the topic, declare the common variables first
      # These two MUST exist
      title   = req.get('topic_title', '')
      description = req.get('topic_description','')
      content = req.get('zforum_post_content', '')
      errors  = []

      if title == '':
        errors.append(self.tr('err_missingttitle', 'Missing Topic Title'))
      if content == '':
        errors.append(self.tr('err_missingtcontent', 'Missing Topic Content'))
      if not can_post:
        errors.append(self.tr('err_no_topic_add', 'You are not authorized to add topics to this forum'))

      if errors:
        req.set('errors', errors)
        return self.add_topic_form(self, REQUEST=req)
      else:
        is_admin = self.zfIsAdmin()
        
        if is_auth:
          # Continue defining global_variables
          add_signature = 1
          if req.get('add_signature', '') == '':
            add_signature = 0

          subscribe_immediately = 1
          if req.get('subscribe_immediately', '') == '':
            subscribe_immediately = 0
        else:
          # If this forum has the (bad, IMHO) allowance of anonymous postings,
          # default some variables oherwise available as choice to logged in users..
          add_signature = 0
          subscribe_immediately = 0
         
        # ADMIN DEFINED VARIABLES
        if is_admin:
        
          locked_flag = 1
          if req.get('locked_flag', '') == '':
            locked_flag = 0

          sticky_flag = 1
          if req.get('sticky_flag', '') == '':
            sticky_flag = 0

          system_announcement_flag = 1
          if req.get('system_announcement_flag', '') == '':
            system_announcement_flag = 0

          creation_user = req.get('creation_user', '')
          if creation_user == '':
            creation_user = user.getUserName()
            
          modifying_user = req.get('modifying_user', '')
          if modifying_user == '':
            modifying_user = user.getUserName()

          creation_date = req.get('creation_date', '')
          if creation_date == '':
            creation_date = DateTime().ISO()
          else:
            try:
              creation_date = DateTime(creation_date).ISO()
            except:
              creation_date = DateTime().ISO()
          
          modifying_date = req.get('modifying_date', '')
          if modifying_date == '':
            modifying_date = DateTime().ISO()
          else:
            try:
              modifying_date = DateTime(modifying_date).ISO()
            except:
              modifying_date = DateTime().ISO()
      
        else:
          locked_flag = 0
          sticky_flag = 0
          system_announcement_flag = 0
          creation_user = user.getUserName()
          creation_date = DateTime().ISO()
          modifying_user = user.getUserName()
          modifying_date = DateTime().ISO()
          
        forum = self.zSQL.getForum(forum_id=forum_id)[0]

        # Disabled flag is used mostly for forums that are moderated..
        disabled_flag = 0
        if forum.moderation_flag:
          disabled_flag = 1
          if is_admin:
            disabled_flag = 0

        if add_signature:
          signature = self.zfGetUserProperty('zfmp_signature', user.getUserName(), '')
          if signature != '':
            new_content = '%s\r\n\r\n[signature]\n%s\n[/signature]' % (content, signature)
            content = new_content

        # For storing the IP address
        if req.get('HTTP_X_FORWARDED_FOR', '') != '':
          ip_address = req['HTTP_X_FORWARDED_FOR']
        else:
          ip_address = req['REMOTE_ADDR']
		
        # Here we have all that is needed to add this topic
        topic_id = self.zSQL.addTopic(forum_id=forum_id,
                                      title=title,
                                      content=content,
                                      description=description,
                                      creation_user=creation_user,
                                      creation_date=creation_date,
                                      locked_flag=locked_flag,
                                      disabled_flag=disabled_flag,
                                      sticky_flag=sticky_flag,
                                      poll_flag=0,
                                      system_announcement_flag=system_announcement_flag)[0].topic_id
        post_id = self.zSQL.addPost(topic_id=topic_id,
                                    content=content,
                                    creation_user=creation_user,
                                    creation_date=creation_date,
                                    modifying_user=modifying_user,
                                    modifying_date=modifying_date,
                                    ip_address=ip_address)[0].post_id
        self.zSQL.updatePostIndexes(topic_id=topic_id, last_post_id=post_id, first_post_id=post_id)        
        if subscribe_immediately:
          self.zSQL.addMemberSubscription(acl_user=user.getUserName(), object_id=topic_id, obj_type='t')

        # Now here I need to send an email to any members that are subscribed to this forum
        self.zfUpdateSubscribers(forum_id, topic_id, content)
 
        # Add a new posting to the user's posting count
        if is_auth:
          current_hits = int(self.zfGetUserProperty('zfmp_postings', creation_user, 0))
          self.zfPutUserProperty(creation_user, 'zfmp_postings', current_hits + 1)

      if friendly_urls:
        url = '%s/view_topic/%s/0' % (self.zfForumBase(), topic_id)
      else:
        url = '%s/view_topic?topic_id=%s' % (self.zfForumBase(), topic_id)

      return req.RESPONSE.redirect(url)

    
  # Function:zfAddTopicResponse:Handles the addition of a topic response
  security.declarePublic('zfAddTopicResponse')
  def zfAddTopicResponse(self):
    """ Handles the addition of a topic response """
    req       = self.REQUEST
    forum_id  = req['forum_id']
    topic_id  = req['topic_id']
    content   = req.get('zforum_post_content', '')
    user      = req['AUTHENTICATED_USER']
    username  = user.getUserName()
    preview   = req.get('preview', '')
    forum     = self.zSQL.getForum(forum_id=forum_id)[0]
    # Topic response default flags
    title         = '' # This can change in future versions, right now, there are no titles for responses
    locked_flag   = 0
    disabled_flag = 0
    sticky_flag   = 0
    hidden_flag   = 0
    poll_flag     = 0
    system_announcement_flag = 0
    today_date    = DateTime().ISO()

    # For storing the IP address
    if req.get('HTTP_X_FORWARDED_FOR', '') != '':
      ip_address = req['HTTP_X_FORWARDED_FOR']
    else:
      ip_address = req['REMOTE_ADDR']

    add_signature = req.get('add_signature', '')

    # This will generate a list of roles that the user must have (at least one)
    # in order to be allowed to successfully add a response to the posting
    posting_min_roles = forum.add_postings_access_roles.split(',')
    allow_posting = 0
    for role in user.getRoles():
      if role in posting_min_roles:
        allow_posting = 1
        break

    if preview:
      req.set('add_comment', 1)
      return self.view_topic(self, REQUEST=req)
    else: #request['add']
      # This will prevent people try to add postings without proper authorization
      if allow_posting:
        if content:
          if add_signature:
            new_content = '%s\r\n\r\n[signature]\n%s\n[/signature]' % \
              (content, self.zfGetUserProperty('zfmp_signature', username, ''))
            content = new_content
          post_id = self.zSQL.addPost(topic_id=topic_id,
                                    content=content,
                                    creation_user=username,
                                    creation_date=today_date,
                                    modifying_user=username,
                                    modifying_date=today_date,
                                    ip_address=ip_address)[0].post_id
          self.zSQL.updatePostIndexes(topic_id=topic_id, last_post_id=post_id)

        # Script Gets here, it means that we need to update all possible subscribers of this topic
        subscription_code = self.zfUpdateSubscribers(forum_id=forum_id, topic_id=topic_id, content=content)
        # Add one hit to this member
        # Maybe this is a good place to add the "rating" because at this time. We know how many postings the member has
        new_postings = int(self.zfGetUserProperty('zfmp_postings', username, 0)) + 1
        self.zfPutUserProperty(username, 'zfmp_postings', new_postings)
        # redirect to the last post in topic
      redirect_url = self.zfLastPostAnchor(topic_id)
      # Old redirect
      #return req.RESPONSE.redirect('%s/view_topic?topic_id=%s&scode=%s#add_comment' % (self.zfForumBase(), topic_id, subscription_code))
      return req.RESPONSE.redirect(redirect_url)
  
  # Function zfEditPostsAndTopics(request): Handles the edition (changing) of a post or topic
  security.declarePublic('zfEditPostsAndTopics')
  def zfEditPostsAndTopics(self):
    """ Handles the edition of a post or topic """
    req = self.REQUEST
    post_id   = req['post_id']
    post      = self.zSQL.getPost(post_id=post_id)[0]
    topic_id  = post.topic_id
    topic     = self.zSQL.getTopic(topic_id=topic_id)[0]
    forum_id  = topic.forum_id
    forum     = self.zSQL.getForum(forum_id=forum_id)[0]
    # Editing options
    preview   = req.get('preview', '')
    save      = req.get('save', '')
    remove    = req.get('remove', '')
    is_admin  = self.zfIsAdmin()
    is_moder  = self.zfIsModerator()
    topic_flag    = post.post_id == topic.topic_first_post_id
    friendly_urls  = self.zfGetSystemProperty('zfsp_friendly_url', '')
    errors = []
    
    if preview:
      req.set('add_comment', 1)
      return self.edit_topic_form(self, REQUEST=req)
    
    elif remove:
      if is_admin or is_moder:
        url = self.zfDeleteTopic(post=post, topic=topic, forum=forum)
        if not url:
          errors.append('ERROR: in zfEditTopic method while deleting post or topic')
          req.set('errors', errors)
          return self.edit_topic_form(self, REQUEST=req)
      else:
        # This would be no good, someone may be attempting to hack it
        if friendly_urls:
          url = '%s/view_topic/%s/0' % (self.zfForumBase(), topic_id)
        else:
          url = '%s/view_topic?topic_id=%s' % (self.zfForumBase(), topic_id)
      return req.RESPONSE.redirect(url)
    
    elif save:  
      # Is user allowed to post?
      allow_editing = self.zfCanEditTopic(post.creation_user)
      if allow_editing:
        errors = self.zfCheckPostingData(topic_flag)
        if errors:
          req.set('errors', errors)
          return self.edit_topic_form(self, REQUEST=req)        
        if topic_flag:
          self.zfEditTopics(topic)
        self.zfEditPost(post)

        subscription_code = self.zfUpdateSubscribers(forum_id=forum_id, topic_id=topic_id, content='One post At this topic was edited')
      # IMO Frendly URL should goes here
      # I'm not sure to send modyfied content by this or only the info about update
      return req.RESPONSE.redirect('%s/view_topic?topic_id=%s&scode=%s#add_comment' % (self.zfForumBase(), topic_id, subscription_code))

  # Function zfEditTopic(request): Handles the edition (changing) of a post or topic
  security.declarePrivate('zfEditTopics')
  def zfEditTopics(self, topic):
    """ Handles the edition of a topic """
    req = self.REQUEST
    is_admin  = self.zfIsAdmin()
    is_moder  = self.zfIsModerator()
    new_topic_title   = req.get('new_topic_title', '')
    topic_description = req.get('topic_description', '')
    # Reasign to forum only for admins and moders
    if is_admin or is_moder:
      new_forum_id = req.get('new_forum_id', topic.forum_id)
    else:
      new_forum_id = topic.forum_id
    # Admins can change other flags: locked, sticky, system, disabled
    if is_admin:
      # Locked Flag
      if req.get('locked_flag', '') == '':
        locked_flag = 0 
      else:
        locked_flag = 1
      # Sticky Flag
      if req.get('sticky_flag', '') == '':
        sticky_flag = 0
      else:
        sticky_flag = 1
      # System Announcement Flag
      if req.get('system_announcement_flag', '') == '':
        system_announcement_flag = 0
      else:
        system_announcement_flag = 1
      # Disabled Flag
      if req.get('disabled_flag', '') == '':
        disabled_flag = 0
      else:
        disabled_flag = 1
      creation_user = req.get('creation_user', '')
      creation_date = DateTime(req.get('creation_date', '')).ISO()
    else:
      locked_flag = topic.locked_flag
      sticky_flag = topic.sticky_flag
      system_announcement_flag = topic.system_announcement_flag
      disabled_flag = topic.disabled_flag
      creation_user = topic.creation_user
      creation_date = topic.creation_date
    # Subscribing Option
    subscribe_immediately = 1
    if req.get('subscribe_immediately', '') == '':
       subscribe_immediately = 0
    if subscribe_immediately:
       self.zSQL.addMemberSubscription(acl_user=username, object_id=topic_id, obj_type='t')
    self.zSQL.updateTopic(topic_id=topic.topic_id,
                                new_forum_id=new_forum_id,
                                new_topic_title=new_topic_title,
                                new_topic_desc=topic_description,
                                creation_user=creation_user,
                                creation_date=creation_date,
                                locked_flag=locked_flag,
                                disabled_flag=disabled_flag,
                                sticky_flag=sticky_flag,
                                system_announcement_flag=system_announcement_flag)

  # Function zfEditPost(request): Handles the edition (changing) of a post
  security.declarePrivate('zfEditPost')
  def zfEditPost(self, post):
    """ Handles the edition of a post """
    req = self.REQUEST
    is_admin  = self.zfIsAdmin()
    is_moder  = self.zfIsModerator()
    errors = []
    # Forum Role Check       
    if is_admin:
      creation_user = req.get('creation_user', '')
      creation_date = DateTime(req.get('creation_date', '')).ISO()
      modifying_user = req.get('modifying_user', '')
      modifying_date = DateTime(req.get('modifying_date', '')).ISO()
    elif is_moder:
      creation_user = post.creation_user
      creation_date = post.creation_date
      modifying_user = req['AUTHENTICATED_USER'].getUserName()
      modifying_date = DateTime().ISO()
    else:
      creation_user = post.creation_user
      creation_date = post.creation_date
      modifying_user = post.creation_user
      modifying_date = DateTime().ISO()
    content = req.get('zforum_post_content', '')
    # If Creation user was changed change the IP of post
    if creation_user != post.creation_user:
      if req.get('HTTP_X_FORWARDED_FOR', '') != '':
        ip_address = req['HTTP_X_FORWARDED_FOR']
      else:
        ip_address = req['REMOTE_ADDR']
    else:
      ip_address = post.ip_address
    # Content procesing gose here
    original_content = post.content  
    add_signature = req.get('add_signature', '')
    if add_signature:
      user_signature = self.sql.zfGetUserProperty('signature', username, '')
      if user_signature:
        new_content = '%s\r\n\r\n[signature]\n%s\n[/signature]' % (content, user_signature)
        content = new_content
    # Good to go, now, on a topic edition situation, I cannot just overwrite the
    # "original posting", I will hide it instead and allow anyone who cares to view it..
    # NOTE: This does not apply to Admins and moderators
    if is_admin:
      proc_content = content
    elif is_moder:
      proc_content = '[i]%s: %s[/i]\n\n%s' % (DateTime().strftime(self.tr('global_date', '%m/%d/%Y')),
                                              self.tr('topic_moder_edit', 'Message has been edited by the forum moderator'),
                                              content)                                     
    else:
    # Replace any previous editings with a nested pre..
      original_content = original_content.replace('[hidden]', '[quote]')
      original_content = original_content.replace('[/hidden]', '[/quote]')
      proc_content = '[hidden][b]%s - [[i]%s[/i]][/b]\n%s[/hidden]\n%s' % \
               (self.tr('global_origmsg', 'Original Message'),
               DateTime().strftime(self.tr('global_date', '%m/%d/%Y')),
               original_content, content)
    self.zSQL.updatePost(post_id=post.post_id,
                         content=proc_content,
                         creation_user=creation_user,
                         creation_date=creation_date,
                         modifying_user=modifying_user,
                         modifying_date=modifying_date,
                         ip_address=ip_address)
                                 
  # Function zfDeleteTopic(request):Handles the removing of a topic/post
  # This is private method so security procedures goes outside of this method
  security.declarePrivate('zfDeleteTopic')
  def zfDeleteTopic(self, post, topic, forum):
    """ Handles the removing of a topic """
    req            = self.REQUEST
    referer        = req['HTTP_REFERER']
    post_id        = post.post_id
    topic_id       = topic.topic_id
    forum_id       = forum.forum_id
    friendly_urls  = self.zfGetSystemProperty('zfsp_friendly_url', '')
    #Here goes security checks of integrity paramers
    if post.topic_id == topic.topic_id and topic.forum_id == forum.forum_id:
      parent_flag     = post_id == topic.topic_first_post_id
      topic_last_flag = post_id == topic.topic_last_post_id 
      forum_last_flag = post_id == forum.forum_last_post_id
      if parent_flag:
        topic_posts = topic.topic_replies + 1
        self.zSQL.delTopicPosts(topic_id=topic_id)
        self.zSQL.delTopic(topic_id=topic_id)
        if topic.topic_last_post_id == forum.forum_last_post_id:
          latest_post = self.zSQL.getLatestPostFromForum(forum_id=forum_id)
          if latest_post:
            latest_post_id = latest_post[0].post_id
          else:
            latest_post_id = 0
          self.zSQL.updateForumAfterPostDelete(forum_id=forum_id, posts=topic_posts, topic=1, new_last=latest_post_id)
        else:
          self.zSQL.updateForumAfterPostDelete(forum_id=forum_id, posts=topic_posts, topic=1)
        if friendly_urls:
          url = '%s/view_forum/%s/0' % (self.zfForumBase(), forum_id)
        else:
          url = '%s/view_forum?forum_id=%s' % (self.zfForumBase(), forum_id)
      else:
        self.zSQL.delPost(post_id=post_id)
        if post_id == topic.topic_last_post_id:
          latest_post_id = self.zSQL.getLatestPostFromTopic(topic_id=topic_id)[0].post_id
          self.zSQL.updateTopicAfterPostDelete(topic_id=topic_id, new_last=latest_post_id)
        else:
          self.zSQL.updateTopicAfterPostDelete(topic_id=topic_id)
        if post_id == forum.forum_last_post_id:
          latest_post_id = self.zSQL.getLatestPostFromForum(forum_id=forum_id)[0].post_id
          self.zSQL.updateForumAfterPostDelete(forum_id=forum_id, new_last=latest_post_id)
        else:
          self.zSQL.updateForumAfterPostDelete(forum_id=forum_id)
        if friendly_urls:
          url = '%s/view_topic/%s/0' % (self.zfForumBase(), topic_id)
        else:
          url = '%s/view_topic?topic_id=%s' % (self.zfForumBase(), topic_id)
      if referer.find('zAdmin') >= 0:
        url = '%s/forums_html' % (self.zAdmin.absolute_url()) 
      return url          
    else:
      return None 

  # Function zfCheckPostingData(request): Handles chacking for errors in post data
  security.declarePrivate('zfCheckPostingData')
  def zfCheckPostingData(self, topic_flag):
    """ Handles the error check for posting """
    req       = self.REQUEST
    is_admin  = self.zfIsAdmin()
    errors    = []
    creation_user  = req.get('creation_user', '')
    creation_date  = req.get('creation_date', '')
    modifying_user = req.get('modifying_user', '')
    modifying_date = req.get('modifying_date', '')
    content        = req.get('zforum_post_content', '')
    topic_title    = req.get('new_topic_title', '')
    if topic_flag and topic_title == '':
      errors.append(self.tr('err_missingttitle', 'Missing Topic Title'))
    if content == '':
      errors.append(self.tr('err_missingtopicresp', 'Missing Topic Response'))
    if is_admin:
      if creation_user == '':
        errors.append(self.tr('err_missingcuser', 'Missing Creation User'))
      if modifying_user == '':
        errors.append(self.tr('err_missingmuser', 'Missing Modifying User'))
      if creation_date == '':
        errors.append(self.tr('err_missingcdate', 'Missing Creation Date'))
      else:
        try:
          d_creation_date = DateTime(creation_date).ISO()
        except:
          errors.append(self.tr('err_invalidcdate', 'Invalid Creation Date'))
      if modifying_date == '':
        errors.append(self.tr('err_missingmdate', 'Missing Modifying Date'))
      else:
        try:
          d_modifying_date = DateTime(modifying_date).ISO()
        except:
          errors.append(self.tr('err_invalidmdate', 'Invalid Modifying Date'))
    if errors:
      return errors
    else:
      return None
      
  # Function:zfSimpleSendMessage(mto, mfrom, subject, body):Main module for sending internal emails
  security.declarePublic('zfSimpleSendMessage')
  def zfSimpleSendMessage(self, mto, mfrom, subject, body):
    # Return Codes:
    # 0 - Found  Normally Via zForum MailHost
    # 1 - Found Normally Via 3rd Party Mailhost (Plone, acquired one, etc)
    # 2 - MailHost Not found Local
    # 3 - MailHost Not found External
    # 4 - Error on Mail Server return
    # 5 - User Cancelled Message (Not getting to this module)
    try:
      mailhost = getattr(self, self.superValues('Mail Host')[0].id)
      rval = 0
    except:
      mailhost = None
      rval = 2

    if not mailhost:
      # Hmm, I need to "try" to acquire any mail host that may
      # exist in the acquisition path, I know I can use the __of__
      # to wrap a container from a class, in this case is self, I
      # wonder if I am not alucinating here, the incredible thing
      # is that it appears to work......
      acq_wrapper = self.__of__(self)
      try:
        mailhost = getattr(acq_wrapper, acq_wrapper.superValues('Mail Host')[0].id)
        rval = 1
      except:
        mailhost = None
        rval = 3

    if mailhost:
      try:
        messageText  = 'From: "zForum Website" <%s>\n' % mfrom
        messageText += 'To: "zForum Member " <%s>\n' % mto
        messageText += 'Subject: %s\n' % subject
        messageText += 'Date: %s\n\n' % DateTime().rfc822()
        messageText += '%s\n\n%s' % (body, self.zfForumBase())
        mailhost.send(messageText)
        rval = 0
      except: # Connection Refused, Invalid Strings, etc..
        rval = 4
    
    return rval



  # Function:zfCanViewForum(forum_id):Tells if the user is allowed to view topics in a forum
  security.declarePublic('zfCanViewForum')
  def zfCanViewForum(self, forum_id):
    """ Tells if the user is allowed to view topics in a forum """
    req = self.REQUEST
    user = req['AUTHENTICATED_USER']
    allowed = 0    
    if user.has_permission('Admin zForums', self):
      allowed = 1
    else:
      forum = self.zSQL.getForum(forum_id=forum_id)
      if forum:
        if user.has_role(['Anonymous']):
          if forum[0].anonymous_viewaccess:
            allowed = 1
        else:
          if [role for role in forum[0].add_postings_access_roles.split(',') if user.has_role(role)]:
            allowed = 1
    return allowed
  
  # Function:zfCanPost(forum_id):Tells if the user is allowed to create/reply topics in a forum
  security.declarePublic('zfCanPost')
  def zfCanPost(self, forum_id):
    """ Tells if the user is allowed to create/reply topics in a forum """
    req  = self.REQUEST
    user = req['AUTHENTICATED_USER']
    posting_allowed = 0
    if user.has_permission('Admin zForums', self):
      # Basically, any admin can add/reply postings anywhere
      posting_allowed = 1
    else:
      # Here it gets a little bit more "complicated", I need to extract the forum's
      # allowed roles and compare them against the user's.
      forum = self.zSQL.getForum(forum_id=forum_id)[0]
      forum_roles = forum.add_postings_access_roles.split(',')
      if user.has_role(['Anonymous']):
        if 'Anonymous' in forum_roles:
          if forum.anonymous_viewaccess == 1:
            posting_allowed = 1
      else:
        if [role for role in forum_roles if user.has_role(role)]:
          posting_allowed = 1
        #for role in forum_roles:
        #  if user.has_role(role):
        #    posting_allowed = 1
        #    break
    return posting_allowed

  # Function:zfCanEditTopic(creation_user):Defines if the user can edit a topic
  security.declarePublic('zfCanEditTopic')
  def zfCanEditTopic(self, creation_user):
    """ Defines if the user can edit a topic """
    req  = self.REQUEST
    user = req['AUTHENTICATED_USER']
    edit_allowed = 0
    if (user.has_role('Authenticated') and user.getUserName() == creation_user) or user.has_permission('Admin zForums', self) or user.has_permission('Moderator zForums', self):
      edit_allowed = 1
    return edit_allowed  
  # Function:zfMemberSubscribed(forum_topic_id, object_type):Receives a forum or topic ID and it's definition ('Forum', 'Topic') and tells if the user is subscribed to it
  security.declarePublic('zfMemberSubscribed')
  def zfMemberSubscribed(self, forum_topic_id, object_type):
    """ Receives a forum or topic ID and it's definition ('Forum', 'Topic') and tells if the user is subscribed to it """
    req  = self.REQUEST
    user = req['AUTHENTICATED_USER']
    o_type = object_type[0].lower()
    is_subscribed = 0
    subscription_test = self.zSQL.testSubscription(forum_topic_id=forum_topic_id, o_type=o_type, acl_user=user.getUserName())
    if subscription_test:
      is_subscribed = 1
    return is_subscribed

  # Function:zfGetTopicSubscriptionsAsList(username):Returns a list of topic_id for user subscrptions
  security.declarePublic('zfGetTopicSubscriptionsAsList')
  def zfGetTopicSubscriptionsAsList(self, username):
    """ Returns a dict of topic_id/topic_title for user subscrptions """
    topics = self.zSQL.getTopicSubscriptions(username=username)
    subscriptions = []
    if topics:
      for topic in topics:
        subscriptions.append(topic.topic_id)
    return subscriptions
    
  # Function:zfGetTopicSubscriptions(username):Returns a dict of topic_id/topic_title for user subscrptions
  security.declarePublic('zfGetTopicSubscriptions')
  def zfGetTopicSubscriptions(self, username):
    """ Returns a dict of topic_id/topic_title for user subscrptions """
    topics = self.zSQL.getTopicSubscriptions(username=username)
    subscriptions = []
    if topics:
      for topic in topics:
        subscriptions.append({'topic_id': topic.topic_id, 'topic_title': topic.topic_title})
    return subscriptions
    
  # Function:zfGetForumSubscriptions(username):Returns a dict of forum_id/forum_title for user subscrptions
  security.declarePublic('zfGetForumSubscriptions')
  def zfGetForumSubscriptions(self, username):
    """ Returns a dict of forum_id/forum_title for user subscrptions """
    forums = self.zSQL.getForumSubscriptions(username=username)
    subscriptions = []
    if forums:
      for forum in forums:
        subscriptions.append({'forum_id': forum.forum_id, 'forum_title': forum.forum_title})
    return subscriptions


  # The next 4 methods *could* be joined into one with minimal efforts,
  # I've decided to split them for code clarity sake

  # Function:zfSubscribeForum(forum_id):Handles the subscription of a forum, auto-detects the username
  security.declarePublic('zfSubscribeForum')
  def zfSubscribeForum(self, forum_id):
    """ Handles the subscription of a forum """
    req = self.REQUEST
    user = req['AUTHENTICATED_USER']
    friendly_urls = self.zfGetSystemProperty('zfsp_friendly_url', '')
    if user.has_role('Authenticated'):
      # May need to check wheather you are already subscribed...
      self.zSQL.addMemberSubscription(acl_user=user.getUserName(), object_id=forum_id, obj_type='f')
    if friendly_urls:
      url = '%s/view_forum/%s/0' % (self.zfForumBase(), forum_id)
    else:
      url = '%s/view_forum?forum_id=%s' % (self.zfForumBase(), forum_id)
    return req.RESPONSE.redirect(url)

  # Function:zfUnSubscribeForum(forum_id):Handles the unsubscription of a forum, auto-detects the username
  security.declarePublic('zfUnSubscribeForum')
  def zfUnSubscribeForum(self, forum_id):
    """ Handles the unsubscription of a forum """
    req = self.REQUEST
    user = req['AUTHENTICATED_USER']
    friendly_urls = self.zfGetSystemProperty('zfsp_friendly_url', '')
    if user.has_role('Authenticated'):
      # May need to check wheather you are already subscribed...
      self.zSQL.delMemberSubscription(acl_user=user.getUserName(), object_id=forum_id, obj_type='f')
    if friendly_urls:
      url = '%s/view_forum/%s/0' % (self.zfForumBase(), forum_id)
    else:
      url = '%s/view_forum?forum_id=%s' % (self.zfForumBase(), forum_id)
    return req.RESPONSE.redirect(url)
    
  # Function:zfSubscribeTopic(topic_id):Handles the subscription of a topic, auto-detects the username
  security.declarePublic('zfSubscribeTopic')
  def zfSubscribeTopic(self, topic_id):
    """ Handles the subscription of a topic """
    req = self.REQUEST
    user = req['AUTHENTICATED_USER']
    friendly_urls = self.zfGetSystemProperty('zfsp_friendly_url', '')
    if user.has_role('Authenticated'):
      # May need to check wheather you are already subscribed...
      self.zSQL.addMemberSubscription(acl_user=user.getUserName(), object_id=topic_id, obj_type='t')
    if friendly_urls:
      url = '%s/view_topic/%s/0' % (self.zfForumBase(), topic_id)
    else:
      url = '%s/view_topic?topic_id=%s' % (self.zfForumBase(), topic_id)
    return req.RESPONSE.redirect(url)
    
  # Function:zfUnSubscribeTopic(topic_id):Handles the unsubscription of a topic, auto-detects the username
  security.declarePublic('zfUnSubscribeTopic')
  def zfUnSubscribeTopic(self, topic_id):
    """ Handles the unsubscription of a topic """
    req = self.REQUEST
    user = req['AUTHENTICATED_USER']
    friendly_urls = self.zfGetSystemProperty('zfsp_friendly_url', '')
    if user.has_role('Authenticated'):
      # May need to check wheather you are already subscribed...
      self.zSQL.delMemberSubscription(acl_user=user.getUserName(), object_id=topic_id, obj_type='t')
    if friendly_urls:
      url = '%s/view_topic/%s/0' % (self.zfForumBase(), topic_id)
    else:
      url = '%s/view_topic?topic_id=%s' % (self.zfForumBase(), topic_id)
    return req.RESPONSE.redirect(url)


  # Function:zfIsAdmin:Returns a flag that tells whether the user is a zForum Administrator
  security.declarePublic('zfIsAdmin')
  def zfIsAdmin(self):
    req = self.REQUEST
    user = req['AUTHENTICATED_USER']
    return user.has_permission('Admin zForums', self)
  
  # Function:zfIsUserAdmin(username):Returns a flag that tells whether any user is a zForum Administrator
  security.declarePublic('zfIsUserAdmin')
  def zfIsUserAdmin(self, username):
    user = self.acl_user.getUser(username)
    return user.has_permission('Admin zForums', self)
  
  # Function:zfIsModerator: Returns a flag that tells whether the user is a zForum Moderator
    security.declarePublic('zfIsModerator')
  def zfIsModerator(self):
    req = self.REQUEST
    user = req['AUTHENTICATED_USER']
    return user.has_permission('Moderator zForums', self)
    
  # Function: zfIsUserModerator(username): Returns a flag that tells whether any user is a zForum Moderator
    security.declarePublic('zfIsUserModerator')
  def zfIsUserModerator(self, username):
    user = self.acl_user.getUser(username)
    return user.has_permission('Moderator zForums', self)
    
  # Function:zfGetUserNames():Return a list of usernames
  security.declarePublic('zfGetUserNames')
  def zfGetUserNames(self):
    auth_user = self.REQUEST['AUTHENTICATED_USER'].has_role(['Authenticated'])
    # Grab all the usernames and return it as a list
    if auth_user:
      if hasattr(self, 'acl_users'):
        users = self.acl_users.getUserNames()
      else:
        # No acl_users, try to acquire it
        acq_wrapper = self.__of__(self)
        if hasattr(acq_wrapper, 'acl_users'):
          users = acq_wrapper.acl_users.getUserNames()
        else:
          users = []
    else:
      users = []
    return users

  # Function zfGenPwd():Generates a somewhat easy-to-read humanoid pseudo-random temporary password
  security.declarePublic('zfGenPwd')
  def zfGenPwd(self):
    vowels='aeiou'
    consonants='bcdfghjklmnpqrstvwxyz'
    password=''
    # Some default values should be in the parameters section
    minpairs = 4
    maxpairs = 6

    for x in range(1,random.randint(int(minpairs),int(maxpairs))):
      consonant = consonants[random.randint(1,len(consonants)-1)]
      if random.choice([1,0]):
        consonant=string.upper(consonant)
      password=password + consonant
      vowel = vowels[random.randint(1,len(vowels)-1)]
      if random.choice([1,0]):
        vowel=string.upper(vowel)
      password=password + vowel
      
    return password

  # Function:zfSignup:Handles Submission for New Membership
  security.declarePublic('zfSignup')
  def zfSignup(self):
    """ Handles Submission for New Membership """
    req = self.REQUEST

    # Will we require to gnerate the password automatically?
    zfsp_email_pwd_on_signup = self.zfGetSystemProperty('zfsp_email_pwd_on_signup', '')    

    # Variable Declaration:
    qs = ''
    new_username = req['new_username']
    
    if zfsp_email_pwd_on_signup:
      qs = '?email_pwd=1'
      new_password = self.zfGenPwd()
      new_password_confirm = new_password
    else:
      new_password             = req['new_password']
      new_password_confirm     = req['new_password_confirm']
      
    user_real_name           = req['user_real_name']
    user_email               = req['user_email']
    errors                   = []
    restricted_usernames     = ['manager', 'root', 'administrator', 'admin']
    roles                    = ['zMember']

    # Error Checking (with somewhat optimized for the username uniquness verification)
    if new_username == '':
      errors.append(self.tr('err_missinguname', 'Missing Username'))  
    if new_password == '':
      errors.append(self.tr('err_missingpwd', 'Missing Password'))
    if new_password_confirm == '':
      errors.append(self.tr('err_missingpwdc','Missing Password Confirmation'))
      
    if not errors:
      if new_password != new_password_confirm:
        errors.append(self.tr('err_pwdandconf', 'Password and Password Confirmation do not match'))
      if new_username.lower() in restricted_usernames:
        errors.append(self.tr('err_userrestricted', 'Usename is a Restricted one. Please choose another'))
      else:
        # At this point all we need to verify is that the username has not been taken,
        # I did the error checking this way to prevent (yet more) overhead in zope
        # In addition, even though the username can be a combination of upper/lower letters,
        # Such as "SpeedBird", I will not allow another user to register with the same name
        # even if capitalization is different.
        if hasattr(self, 'acl_users'):
          acl_users = getattr(self, 'acl_users')
          usernames = self.acl_users.getUserNames()
        else:
          # No acl_users, try to acquire it
          acq_wrapper = self.__of__(self)
          if hasattr(acq_wrapper, 'acl_users'):
            acl_users = getattr(acq_wrapper, 'acl_users')
            usernames = acq_wrapper.acl_users.getUserNames()
          else:
            usernames = []
        for username in usernames:
          uname = username.lower()
          if new_username.lower() == uname:
            errors.append(self.tr('err_usernaneused', 'Username is already taken, please choose another one'))
            break

    if errors:
      req.set('errors', errors)
      return self.signup_form(self, REQUEST=req)
    else:
      # Add the user Here, all is ok
      user_info = {'name': new_username,
                   'password': new_password,
                   'confirm': new_password,
                   'roles': roles,
                   'domains': []}    
      # Note that a user instance must exist on the forum, otherwise this
      # won't do much, in any case, if the administrator opted to use his
      # own auth methods, most likely the "Signup" button (the one that calls
      # this method) would be hidden/removed from the User Interface (UI) anyway.
      try:
        acl_users.manage_users(submit='Add', REQUEST=user_info)
        added = 1
      except:
        added = 0

      # User has been added to zope, now take care of the "extra" properties that might be in the 
      # signup form (currently first name and email)
      if added:
        if user_real_name:
          self.zfPutUserPropertyRestricted(new_username, 'zfmp_username', user_real_name)
        if user_email:
          self.zfPutUserPropertyRestricted(new_username, 'zfmp_email', user_email)
        self.zfPutUserPropertyRestricted(new_username, 'zfmp_join_date', DateTime().ISO())

        # Now send the mail with the pasword if requested
        if zfsp_email_pwd_on_signup:
          mfrom = self.zfGetSystemProperty('zfsp_admin_contact_email', 'noreply@zforum.org')
          subject = "zForum Membership For %s" % user_real_name
          # Body should probably be some kind of dynamic, admin-modifyable text
          body = """Hello %s,
              
You recently subscribed to zForum Forum Systems.

Your user name is: %s
Your temporary password is: %s

Please Log In the system using the password provided, then you can just click on the =Edit Profile= button to change it to your preference.

To Log In, Go To:

%s/index_html

Or, to log in directly, go to:

%s/zfLogin?__ac_name=%s&__ac_password=%s""" % (user_real_name, new_username, new_password, self.zfForumBase(),
                                               self.zfForumBase(), new_username, new_password)
          # In a near version upgrade, try to change the codes teturned to a list/dict
          mail_code = self.zfSimpleSendMessage(mto=user_email, mfrom=mfrom, subject=subject, body=body)        
      return req.RESPONSE.redirect('%s/login_form%s' % (self.zfForumBase(), qs))

  # Function:zfGetMemberAvatarID(username):Grabs a member's avatar ID
  security.declarePublic('zfGetMemberAvatarID')
  def zfGetMemberAvatarID(self, username):
    """ Grabs a member's avatar ID """
    avatar = self.zSQL.getMemberAvatar(username=username)
    if avatar:
      avatar_id = avatar[0].avatar_id
    else:
      avatar_id = {}
    return avatar_id

  # Function:zfGetMemberAvatar(username):Returns the actual image data for an avatar image
  security.declarePublic('zfGetMemberAvatar')
  def zfGetMemberAvatar(self, username):
    """ Retrieves Member Avatar Image """
    avatar = self.zSQL.getMemberAvatarData(username=username)
    if avatar:
      self.REQUEST.RESPONSE.setHeader('content-type', 'image/' + avatar[0].content_type)
      #self.REQUEST.RESPONSE.setHeader('Content-Length', len(avatar[0].avatar_image))
      #self.REQUEST.RESPONSE.write(str(avatar[0].avatar_image))
      return base64.decodestring(avatar[0].avatar_image)
    else:
      pass

  # Function:zfUpdateMemberProperties():Edits basic member user properties
  security.declarePublic('zfUpdateMemberProperties')
  def zfUpdateMemberProperties(self):
    """ Handles User Properties Changes """
    req        = self.REQUEST
    user       =req['AUTHENTICATED_USER']
    username   = user.getUserName()
    is_admin   = user.has_permission('Admin zForums', self)
    other_user = req.get('other_user', '')
    new_roles  = req.get('new_roles', [])
    new_lang   = req.get('new_lang', '')
    is_plone   = req.get('is_plone', '')

    if new_lang == '':
      new_lang = self.zfGetSystemProperty('zfsp_system_language', 'en_US')
      
    if is_admin:
      # This means that this is an admin that might be trying to change somebody else's
      # properties
      if other_user:
        username = other_user

    # Avatar Restrictions (px) - Maybe we need to make these dynamic??
    MAX_HEIGHT = 150
    MAX_WIDTH  = 100
    MAX_SIZE   = 15000 # Bytes
    
    new_password         = req.get('new_password', '')
    new_password_confirm = req.get('new_password_confirm', '')
    user_full_name       = req.get('zfmp_username', '')
    country              = req.get('zfmp_country', '')
    user_email           = req.get('zfmp_email', '')
    url                  = req.get('zfmp_web_page', '')
    avatar_delete        = req.get('avatar_delete', '')
    avatar               = req.get('avatar', '') # FILE Object
    remove_forum_ids     = req.get('forum_ids', []) # Must come as a list
    remove_topic_ids     = req.get('topic_ids', [])
    new_signature        = req.get('zfmp_signature', '')
    errors               = []
    logout               = 0

    if not is_plone:
      # Step 1 - Change Member password
      # Grab the user roles
      if new_roles:
        roles = new_roles
      else:
        auth_roles = user.getRoles()
        roles = []
        for role in auth_roles:
          if role != 'Authenticated':
            roles.append(role)

      if new_password == new_password_confirm and new_password != '':
        # Do want to change it, grab roles
        try:
          # Remember, ALL this functionality is USELESS if your users are authenticated
          # elsewhere, it will NOT break, but I'd suggest remove or hide
          # The password/confirmation fields from the edit user properties page, it wont do
          # any harm..
          user_info = {'name': username,
                       'password': new_password,
                       'confirm': new_password_confirm,
                       'roles': roles}
          self.acl_users.manage_users(submit='Change', REQUEST=user_info)
          logout = 0
        except:
          errors.append(self.tr('global_passwdnochange', 'Password could not be changed, user is probably authenticated in another context'))
      else:
        # Here we have to update the possible roles change
        user_info = {'name': username,
                     'password': 'password',
                     'confirm': 'pconfirm',
                     'roles': roles}
        self.acl_users.manage_users(submit='Change', REQUEST=user_info)
        
        if new_password != '':
          errors.append('Password and confirmation do not match, password was not changed')

    # Email Settings
    self.zfPutUserProperty(username, 'zfmp_email', user_email)

    # Country
    self.zfPutUserProperty(username, 'zfmp_country', country)
      
    # Subscriptions
    if remove_forum_ids:
      for forum_id in remove_forum_ids:
        self.zfUnSubscribeForum(forum_id)
    if remove_topic_ids:
      for topic_id in remove_topic_ids:
        self.zfUnSubscribeTopic(topic_id)

    # User Full Name
    self.zfPutUserProperty(username, 'zfmp_username', user_full_name)

    # Avatars
    if avatar_delete:
      self.zSQL.delMemberAvatar(username=username)
    if avatar and avatar.filename:
      self.zSQL.delMemberAvatar(username=username)
      # Add Avatar Here
      imagedata = avatar.read()
      dummy, content_type = string.split(avatar.headers['Content-Type'], "/")
      
      # Next Line of code allows me to translate the raw image data into an
      # image-like stream
      c_type, width, height = getImageInfo(imagedata)
      
      if height > MAX_HEIGHT or width > MAX_WIDTH:
        errors.append('Image dimensions (%spx width, %spx height) exceeds the limits set by the administrator of (%spx width, %spx height)' % (width, height, MAX_WIDTH, MAX_HEIGHT))
      if len(imagedata) > MAX_SIZE:
        errors.append('Avatar must not exceed %s bytes in size, regardless of its dimensions' % MAX_SIZE)
      if not errors:
        # Starting on zForum 1.0.3, Avatar Data will be stored in "text" via base64 encoding,
        # this is mainly due to compatibility among database systems
        self.zSQL.addMemberAvatar(username=username, avatar=base64.encodestring(imagedata), content_type=content_type)

    # Signature
    self.zfPutUserProperty(username, 'zfmp_signature', new_signature)

    #  URL
    self.zfPutUserProperty(username, 'zfmp_web_page', url)

    # Language/Locale
    self.zfPutUserProperty(username, 'zfmp_locale', new_lang)
    req.RESPONSE.setCookie('zforum_language', new_lang, path='/')

    # Handle errors and/or return back to the UI
    if errors:
      req.set('errors', errors)
      return self.user_prefs_form(self, REQUEST=req)
    else:
      if logout:
        req.RESPONSE.redirect('%s/login_form' % (self.zfForumBase()))
      else:
        if is_admin and other_user != '':
          return req.RESPONSE.redirect('%s/users_html?saved=1' % (self.zAdmin.absolute_url()))
        else:
          if is_plone:
            return req.RESPONSE.redirect('%s/user_prefs_plone_form?saved=1' % (self.zfForumBase()))
          else:
            return req.RESPONSE.redirect('%s/user_prefs_form?saved=1' % (self.zfForumBase()))

  # Function:zfGetRank(postings):Based on the number of postings, return the rank of the member
  security.declarePublic('zfGetRank')
  def zfGetRank(self, postings):
    """ Based on the number of postings, return the rank of the member """
    rank = self.zSQL.getMemberRank(postings=postings)
    if rank:
      rank_label = rank[0].rank_name
    else:
      rank_label = self.tr('global_na', 'n/a')
    return rank_label

  # Function:zfGetInboxUnreadCount():Returns the number of unread messages in the inbox. NOTE Assumes cat_id=1 is the Inbox
  security.declarePublic('zfGetInboxUnreadCount')
  def zfGetInboxUnreadCount(self):
    """ Returns the number of unread messages in the inbox. NOTE Assumes cat_id=1 is the Inbox """
    req      = self.REQUEST
    user     = req['AUTHENTICATED_USER']
    username = user.getUserName()
    cat      = self.zSQL.getMessagesCountByCatINBOX(acl_user=username)
    if cat:
      unread  = cat[0].total_unread
    else:
      unread  = 0
    return unread
    
  # Function:zfGetMessagesCountByCat:Returns the number of messages that each category (inbox, sent, etc) has
  security.declarePublic('zfGetMessagesCountByCat')
  def zfGetMessagesCountByCat(self):
    """ Returns a list of dict containing categories/unread messages """
    req      = self.REQUEST
    user     = req['AUTHENTICATED_USER']
    username = user.getUserName()
    cat_dict = {}
    cat_dict_totals = {}
    cat_list = []

    cats = self.zSQL.getMessagesCountByCat(acl_user=username)
    cats_totals = self.zSQL.getMessagesCountByCat(acl_user=username, grab_unread_only=0)

    if cats:
      for cat in cats:
        cat_id      = cat.cat_id
        cat_name    = cat.cat_name
        cat_unread  = cat.cat_tot
        cat_dict[cat_id] = {'cat_id': cat_id, 'cat_name': cat_name, 'cat_unread': cat_unread}
        
    if cats_totals and cats_totals[0].cat_id:
      for cat in cats_totals:
        cat_id   = cat.cat_id
        cat_name = cat.cat_name
        cat_tot  = cat.cat_tot
        cat_dict_totals[cat_id] = {'cat_id': cat_id, 'cat_name': cat_name, 'cat_tot': cat_tot}

    # Ok, Now I have 2 dict with info that I need to consolidate in one:
    for cat_id in cat_dict.keys():
      current_dict = cat_dict[cat_id]
      current_dict.update({'cat_tot': cat_dict_totals[cat_id].get('cat_tot')})
      cat_list.append(current_dict)
       
    return cat_list

  # Function:zfGetMessages(cat_id):Grabs all the messages for a specific category (inbox, sent, etc)
  security.declarePublic('zfGetMessages')
  def zfGetMessages(self, cat_id):
    """ Returns the messages for the category """
    req      = self.REQUEST
    user     = req['AUTHENTICATED_USER']
    username = user.getUserName()

    return self.zSQL.getAllMessagesFor(cat_id=cat_id, acl_user=username)

  # Function:zfGetQuotaPerc():Handles the return of a % for the amount of messages for the user
  security.declarePublic('zfGetQuotaPerc')
  def zfGetQuotaPerc(self):
    """ Handles the return of a % for the amount of messages for the user """
    req  = self.REQUEST
    user = req['AUTHENTICATED_USER']
    username = user.getUserName()
    msgs_len = self.zSQL.getUserMessagesLength(acl_user=username)
    if msgs_len:
      msgs_size = int(msgs_len[0].msgs_len)
    else:
      msgs_size = 0
    member_quota = self.zfGetSystemProperty('zfsp_member_quota', '50000') 
    if not member_quota:
	  quota = 50000
    else:
      quota = int(member_quota)
      if quota <= 0:
        quota = 0.0000001
    actual_prc = int(msgs_size / float(quota) * 100)

    if actual_prc > 100:
      actual_prc = 100

    # This hack is to take in consideration the width used by the start and end image bars
    # and the cellspacing, cellpadding attres from the container table, just size the perc down
    # a bit
    final_prc = int((actual_prc * 100) / 99.0)

    return {'quota': int(quota), 'fake': final_prc, 'real': actual_prc}

  # Function:zfSendPM()[PROTECTED]:Handles the submission of a new message
  security.declareProtected('Access zForum Personal Messages', 'zfSendPM')
  def zfSendPM(self):
    """ Handles sending a PM to an existing user """
    req = self.REQUEST
    forum_id    = req.get('forum_id', '')
    cat_id      = req.get('cat_id', '')
    new_mto     = req.get('new_mto', '')
    new_subject = req.get('new_subject', '')
    new_msg     = req.get('new_msg', '')
    user        = req['AUTHENTICATED_USER']
    username    = user.getUserName()
    errors      = []

    if new_mto == '':
      errors.append('Missing Receipient')
    if new_subject == '':
      errors.append('Missing Subject')
    if new_msg == '':
      errors.append('Missing Message Contents')

    if errors:
      req.set('errors', errors)
      return self.message_new(self, REQUEST=req)
    else:
      # Does this user exist?
      if new_mto in self.zfGetUserNames():
        # These MUST Exist
        inbox_cat_id = self.zSQL.getMessageCategoryIdFor(cat_name='inbox')
        sent_cat_id = self.zSQL.getMessageCategoryIdFor(cat_name='sent')
        sent_flag = 0
        if inbox_cat_id:
          pm_id = self.zSQL.sendMessage(from_acl_user=username,
                                        acl_user=new_mto,
                                        subject=new_subject,
                                        message=new_msg,
                                        cat_id=inbox_cat_id[0].cat_id)[0].pm_id
          sent_flag = 1
        # Now make a "copy" into the user's sent folder and marked as "read"
        if sent_cat_id:
          pm_id = self.zSQL.sendMessage(from_acl_user=username,
                                        acl_user=username,
                                        subject='[%s %s] - %s' % (self.tr('pm_sento', 'Sent To'), new_mto, new_subject),
                                        message=new_msg,
                                        cat_id=sent_cat_id[0].cat_id)[0].pm_id
          self.zfFlagMessageAsRead(pm_id=pm_id, read_flag=1)
        
        # New starting 1.0.7p (Send an email to the recipient)
        mfrom   = self.zfGetUserProperty('zfmp_email', username, '')
        mto     = self.zfGetUserProperty('zfmp_email', new_mto, '')
        subject = '[%s %s] - %s' % (self.tr('pm_sento', 'Sent To'), new_mto, new_subject)
        body = '%s\n\n%s' % (self.tr('pm_email_disclaimer', 'This is a personal message from the discussion forums'), new_msg)
        self.zfSimpleSendMessage(mto, mfrom, subject, body)
        
        return req.RESPONSE.redirect('%s/messages_html?forum_id=%s&cat_id=%s&sent_flag=%s' % (self.zfForumBase(), forum_id, cat_id, sent_flag))
      else:
        errors.append('Member %s Could not be Found' % new_mto)
        req.set('errors', errors)
        return self.message_new(self, REQUEST=req)

  # Function:zfFlagMessageAsRead(pm_id, read_flag)[PROTECTED]:Flags a message id with a read/unread flag, STUB Function
  security.declareProtected('Access zForum Personal Messages', 'zfFlagMessageAsRead')
  def zfFlagMessageAsRead(self, pm_id, read_flag):
    """ Handles Changing the read/unread flag for a message (PM) """
    self.zSQL.flagMessage(pm_id=pm_id, read_flag=read_flag)
    pass

  # Function:zfMessageOptions()[PROTECTED]:Processes actions like transferring messages between categories and deletion of trash
  security.declareProtected('Access zForum Personal Messages', 'zfMessageOptions')
  def zfMessageOptions(self):
    """ Handles Main Messaging Options """
    req = self.REQUEST

    # Grab All possible Information
    cat_id     = req['cat_id']             # required
    forum_id   = req.get('forum_id', '')   # optional
    moveto     = req.get('moveto', '')     # cat_id to move selected messages to
    ids        = req.get('ids', [])        # Selected IDs (messages) (if any)
    move       = req.get('move', '')       # Move messages from cat_id button was pressed
    emptytrash = req.get('emptytrash', '') # User requested deletion of trash folder

    
    if move: # check if we require to move messages
      moved = 0
      if cat_id != moveto: # Messages requested to move to a DIFFERENT category
        if ids: # Are there any checked messages?
          for id in ids: # yes, so process each one of them
            self.zSQL.moveMessage(pm_id=id, to_cat_id=moveto) # Do the move
            moved = 1
      return req.RESPONSE.redirect('%s/messages_html?cat_id=%s&forum_id=%s&moved=%s' % (self.zfForumBase(), cat_id, forum_id, moved))

    if emptytrash: # Self-explanatory
      trash_id = self.zSQL.getMessageCategoryIdFor(cat_name='trash')
      trashed = 0
      if trash_id:
        self.zSQL.emptyMessageTrash(cat_id=trash_id[0].cat_id)
        trashed = 1
      return req.RESPONSE.redirect('%s/messages_html?cat_id=%s&forum_id=%s&emptytrash=%s' % (self.zfForumBase(), cat_id, forum_id, trashed))

    # Kind of the "catch-if-none" redirect, should never get here actually
    return req.RESPONSE.redirect('%s/messages_html?cat_id=%s&forum_id=%s&action=0' % (self.zfForumBase(), cat_id, forum_id))

  # Function:zfGetPM(pm_id)[PROTECTED]:Returns a properly formatted message for viewing/editing
  security.declareProtected('Access zForum Personal Messages', 'zfGetPM')
  def zfGetPM(self, pm_id):
    """ Returns a properly formatted message for viewing/editing """
    pm = self.zSQL.getMessage(pm_id=pm_id)
    username = self.REQUEST['AUTHENTICATED_USER'].getUserName()

    if pm:
      acl_user = pm[0].acl_user
      # Security Check
      if username == acl_user:
        tmp_dict = {'from': pm[0].from_acl_user,
                    'subject': pm[0].subject,
                    'message': pm[0].message,
                    'creation_date': pm[0].creation_date}
      else:
        # You tryin' to fool the system aren't you :)
        tmp_dic = {}
      return tmp_dict
    else:
      return {}

  # Function:zfSearch(search_string):Searches amongst topics/forums that the user is allowed to access
  security.declarePublic('zfSearch')
  def zfSearch(self, search_string):
    """ Searches topics/forums that the user is allowed to access """
    # Please note that this may not be the ideal way to pass the
    # roles to the search query, I did this to be able to easily
    # test the SQL query directly..
    req = self.REQUEST
    # All this functionality will have to be revamped to provide a more "robust"
    # search experience, for example, you cannot search any word 3 or less letters in
    # length, and this is hardcoded, in addittion no "error message" or "warning"
    # is issued if such event occurs..
    if len(search_string) > 3:
      user = req['AUTHENTICATED_USER']
      roles = ''
      for role in user.getRoles():
        roles += '%s,' % role
      # Basically we need to perform a search on the topics that you are allowed to at least view
      # Some "search_string" sanity checks (thanks paulus)
      search_string = search_string.replace("'", "\\'")
      topics = self.zSQL.searchTopics(user_roles=roles[:-1], search_string=search_string)
      if topics:
        req.set('topics', topics)
    return self.search_results(self, REQUEST=req)

  # Function:zfGetCategory(cat_id):Returns a category object (Brains)
  security.declarePublic('zfGetCategory')
  def zfGetCategory(self, cat_id):
    """ Returns a category object """
    cat = self.zSQL.getCategory(cat_id=cat_id)
    if cat:
      return cat[0]
    else:
      return {}

  # Function:zfAdminSendPM():Sends a message to all members in the forum
  security.declareProtected('Admin zForums', 'zfAdminSendPM')
  def zfAdminSendPM(self):
    """ Handles a global send message to all members of the forum """
    # For Admins only
    req = self.REQUEST
    msubject = req.get('msubject', self.tr('pm_msgadmin', 'Message from the Forum Administrator'))
    mfrom    = req.get('mfrom', 'administrator')
    mmsg     = req.get('mmsg', '')
    idx      = 0
    
    if mmsg:
    
      inbox_cat_id = self.zSQL.getMessageCategoryIdFor(cat_name='inbox')
      sent_cat_id = self.zSQL.getMessageCategoryIdFor(cat_name='sent')
      
      if inbox_cat_id and sent_cat_id:
        inbox_id = inbox_cat_id[0].cat_id
        sent_id  = sent_cat_id[0].cat_id
        for username in self.zfGetUserNames():
          pm_id = self.sql.sendMessage(from_acl_user=mfrom,
                                       acl_user=username,
                                       subject=msubject,
                                       message=mmsg,
                                       cat_id=inbox_id)[0].pm_id
          idx += 1
                                         
        pm_id = self.zSQL.sendMessage(from_acl_user=username,
                                      acl_user=username,
                                      subject='[%s] - %s' % (self.tr('pm_sendall', 'Sent To ALL Members'), msubject),
                                      message=mmsg,
                                      cat_id=sent_id)[0].pm_id

        self.zfFlagMessageAsRead(pm_id=pm_id, read_flag=1)

    return req.RESPONSE.redirect('%s/index_html?idx=%s' % (self.zAdmin.absolute_url(), idx))

  # Function:zfProcessCategory():Changes a unique category's properties
  security.declareProtected('Admin zForums', 'zfProcessCategory')
  def zfProcessCategory(self):
    """ Updates a category """
    req = self.REQUEST
    cat_id = req['cat_id'] # Must Exist
    cat_name = req.get('cat_name', '')
    cat_desc = req.get('cat_desc', '')
    visible  = req.get('visible', [])
    errors   = []

    if cat_name == '':
      errors.append('Category Name Cannot Be Empty')

    if errors:
      req.set('errors', errors)
      return self.zAdmin.edit_category_form(self, REQUEST=req)
    else:
      visible_to = ''
      visible.extend(['zAdministrator', 'zAdministratorStaff'])
      for vis in visible:
        visible_to += '%s,' % (vis)
      visible_to = visible_to[:-1]

      self.zSQL.updateCategory(cat_id=cat_id,
                               cat_name=cat_name,
                               cat_desc=cat_desc,
                               cat_visible_to=visible_to)
      return req.RESPONSE.redirect('%s/edit_category_form?cat_id=%s&updated=1' % (self.zAdmin.absolute_url(), cat_id))

  # Function:zfAddCategory():Adds a new category
  security.declareProtected('Admin zForums', 'zfAddCategory')
  def zfAddCategory(self):
    """ Adds a new category """
    req = self.REQUEST
    cat_name = req.get('cat_name', '')
    cat_desc = req.get('cat_desc', '')
    visible  = req.get('visible', [])
    errors   = []

    if cat_name == '':
      errors.append('Category Name Cannot Be Empty')

    if errors:
      req.set('errors', errors)
      return self.zAdmin.add_category_form(self, REQUEST=req)
    else:
      visible_to = ''
      visible.extend(['zAdministrator', 'zAdministratorStaff'])

      temp_pos = self.zSQL.getCategoryLastPosition()[0].last_pos
      if temp_pos:
        position = int(temp_pos) + 1
      else:
        position = 1
      for vis in visible:
        visible_to += '%s,' % (vis)
      visible_to = visible_to[:-1]

      cat_id = self.zSQL.addCategory(cat_name=cat_name,
                                     cat_desc=cat_desc,
                                     cat_order=position,
                                     cat_visible_to=visible_to)[0].cat_id

      return req.RESPONSE.redirect('%s/categories_html?added=1' % (self.zAdmin.absolute_url()))
    

  # Function zfGetCategories:Returns a simple brain object from the SQL's categories table
  security.declareProtected('Admin zForums', 'zfGetCategories')
  def zfGetCategories(self):
    """ Returns a simple brain object from the SQL's categories table """
    cats = self.zSQL.getCategories()
    if cats:
      categories = cats
    else:
      categories = {}
    return categories
  
  # Function zfChangeCategoryPosition() changes category position
  security.declareProtected('Admin zForums', 'zfChangeCategoryPosition')
  def zfChangeCategoryPosition(self, cat_id, direction):
    """ Changes category position """
    categories = self.zSQL.getCategories()
    new_order = 0
    index = 0
    if categories:
      number_of_categories = len(categories)
      for i in range(number_of_categories):
        if int(categories[i].cat_id) == int(cat_id):
          index = i
          break
      if direction == 'up':
        if index > 0:
          go_down_index = index - 1
          go_up_index = index
          new_order = index
      elif direction == 'down':
        if index < number_of_categories - 1:
          go_down_index = index
          go_up_index = index + 1
          new_order = index + 1
      if new_order:
        # This goes Up 
        self.zfUpdateCategoryOrder(categories[go_up_index], new_order)
        # This goes Down 
        self.zfUpdateCategoryOrder(categories[go_down_index], new_order+1)  
        
  # Function zfChangeForumPosition() changes category position
  security.declareProtected('Admin zForums', 'zfChangeForumPosition')
  def zfChangeForumPosition(self, cat_id, forum_id, direction):
    """ Changes forum position """
    forums = self.zSQL.getForumsByCat(cat_id=cat_id)
    new_order = 0
    index = 0
    if forums:
      number_of_forums = len(forums)
      for i in range(number_of_forums):
        if int(forums[i].forum_id) == int(forum_id):
          index = i
          break
      if direction == 'up':
        if index > 0:
          go_down_index = index - 1
          go_up_index = index
          new_order = index
      elif direction == 'down':
        if index < number_of_forums - 1:
          go_down_index = index
          go_up_index = index + 1
          new_order = index + 1
      if new_order:
        # This goes Up 
        self.zfUpdateForumOrder(forums[go_up_index], new_order)
        # This goes Down 
        self.zfUpdateForumOrder(forums[go_down_index], new_order+1)   
   
  # Updates only Order filed from category
  security.declarePrivate('zfUpdateCategoryOrder')
  def zfUpdateCategoryOrder(self, category, cat_order):
    """ Updates category order """
    self.zSQL.updateCategory(cat_id = category.cat_id,
                             cat_name = category.cat_name,
                             cat_desc = category.cat_desc,
                             cat_order = cat_order,
                             cat_visible_to = category.cat_visible_to)
  
  # Updates only Order filed from category
  security.declarePrivate('zfUpdateForumOrder')
  def zfUpdateForumOrder(self, forum, forum_order):
    """ Updates forum order """
    self.zSQL.updateForum(forum_id = forum.forum_id,
                          cat_id = forum.cat_id,
                          forum_title = forum.forum_title,
                          forum_desc = forum.forum_desc,
                          forum_order = forum_order,
                          moderation_flag = forum.moderation_flag,
                          anonymous_viewaccess = forum.anonymous_viewaccess,
                          add_postings_access_roles = forum.add_postings_access_roles)
   
  # Function zfProcessForum():Updates A Forum's Properties
  security.declareProtected('Admin zForums', 'zfProcessForum')
  def zfProcessForum(self):
    """ Updates a Forum """
    req         = self.REQUEST
    forum_id    = req['forum_id']
    cat_id      = req['cat_id']
    forum_title = req.get('forum_title', '')
    forum_desc  = req.get('forum_desc', '')
    errors      = []
    tmp_moderation_flag       = req.get('moderation_flag', '')
    tmp_anonymous_viewaccess    = req.get('anonymous_viewaccess', '')
    add_postings_access_roles = req.get('add_postings_access_roles', [])

    if forum_title == '':
      errors.append('Forum title must not be an empty string')

    if errors:
      req.set('errors', errors)
      return self.zAdmin.edit_forum_form(self, REQUEST=req)
    else:

      if tmp_anonymous_viewaccess == '':
        anonymous_viewaccess = 0
      else:
        anonymous_viewaccess = 1

      if tmp_moderation_flag == '':
        moderation_flag = 0
      else:
        moderation_flag = 1

      add_postings_access_roles.extend(['zAdministrator', 'zAdministratorStaff'])    
      add_postings_access_roles_str = ",".join(add_postings_access_roles)

      self.zSQL.updateForum(forum_id=forum_id,
                            cat_id=cat_id,
                            forum_title=forum_title,
                            forum_desc=forum_desc,
                            moderation_flag=moderation_flag,
                            anonymous_viewaccess=anonymous_viewaccess,
                            add_postings_access_roles=add_postings_access_roles_str)

      return req.RESPONSE.redirect('%s/edit_forum_form?forum_id=%s&updated=1' % (self.zAdmin.absolute_url(), forum_id))

  # Function zfAddForum():Adds a new forum
  security.declareProtected('Admin zForums', 'zfAddForum')
  def zfAddForum(self):
    """ Adds a new forum """
    req         = self.REQUEST
    cat_id      = req['cat_id']
    forum_title = req.get('forum_title', '')
    forum_desc  = req.get('forum_desc', '')
    errors      = []
    tmp_moderation_flag       = req.get('moderation_flag', '')
    tmp_anonymous_viewaccess    = req.get('anonymous_viewaccess', '')
    add_postings_access_roles = req.get('add_postings_access_roles', [])
    admin_min_access_roles    = req.get('admin_min_access_roles', 'zAdministrator,zAdministratorStaff')

    if forum_title == '':
      errors.append('Forum title must not be an empty string')

    if errors:
      req.set('errors', errors)
      return self.zAdmin.add_forum_form(self, REQUEST=req)
    else:
      if tmp_anonymous_viewaccess == '':
        anonymous_viewaccess = 0
      else:
        anonymous_viewaccess = 1

      if tmp_moderation_flag == '':
        moderation_flag = 0
      else:
        moderation_flag = 1
      temp_pos = self.zSQL.getForumLastPosition(cat_id=cat_id)[0].last_pos
      if temp_pos:
        forum_order = int(temp_pos) + 1
      else:
        forum_order = 1
        
      add_postings_access_roles.extend(['zAdministrator', 'zAdministratorStaff'])
      
      add_postings_access_roles_str = ",".join(add_postings_access_roles)
      
      forum_id = self.zSQL.addForum(cat_id=cat_id,
                                    forum_title=forum_title,
                                    forum_desc=forum_desc,
                                    forum_order=forum_order,
                                    moderation_flag=moderation_flag,
                                    anonymous_viewaccess=anonymous_viewaccess,
                                    add_postings_access_roles=add_postings_access_roles_str,
                                    admin_min_access_roles=admin_min_access_roles)[0].forum_id

      return req.RESPONSE.redirect('%s/forums_html?cat_id=%s' % (self.zAdmin.absolute_url(), cat_id))
      

  # Function:zfProcessForums():Handles the request for addition of a new forum or deletion of an existing ones. Only empty forums can be actually deleted
  security.declareProtected('Admin zForums', 'zfProcessForums')
  def zfProcessForums(self):
    """ Handles the request for addition of a new forum or deletion of an existing ones. Only empty forums can be actually deleted """
    req = self.REQUEST
    add = req.get('add', '')
    cat_id = req.get('cat_id', '')
    remove = req.get('remove', '')
    deleted = 0
    move_up = req.get('up', '')
    move_down = req.get('down', '')
    
    if add:
      return req.RESPONSE.redirect('%s/add_forum_form?cat_id=%s' % (self.zAdmin.absolute_url(), cat_id))
    elif remove:
      # Here I assume tat the IDs passed here will already be validated (i.e.
      # The id's for the forums are from forums that are indeed empty
      ids = req.get('ids', [])
      if ids:
        for forum_id in ids:
          self.zSQL.delForum(forum_id=forum_id)
          deleted += 1
      return req.RESPONSE.redirect('%s/forums_html?cat_id=%s&deleted=%s' % (self.zAdmin.absolute_url(), cat_id, deleted))
    elif move_up:
      self.zfChangeForumPosition(cat_id, move_up, 'up')
      return req.RESPONSE.redirect('%s/forums_html?cat_id=%s' % (self.zAdmin.absolute_url(), cat_id))
    elif move_down:
      self.zfChangeForumPosition(cat_id, move_down, 'down')
      return req.RESPONSE.redirect('%s/forums_html?cat_id=%s' % (self.zAdmin.absolute_url(), cat_id))
    else:
      # This is supposed to be one of those pieces of code that never gets called :)
      return req.RESPONSE.redirect('%s/forums_html?cat_id=%s' % (self.zAdmin.absolute_url(), cat_id))
  
  security.declareProtected('Admin zForums', 'zfGetTopicUpdateLog')
  def zfGetTopicUpdateLog(self, offset):
    limit = int(self.zfGetSystemProperty('zfsp_responses_per_page', 10))
    log = self.zSQL.getTopicsUpdate(offset=offset, limit=limit)
    today = DateTime()
    log_list = []
    for topic in log:
      if (today - topic.modifying_date) <= 1.0:
        update_flag = 1
      else:
        update_flag = 0
      log_list.append({'topic_id':       topic.topic_id,
                       'topic_title':    topic.topic_title,
                       'modifying_date': topic.modifying_date,
                       'modifying_user': topic.modifying_user,
                       'topic_replies':  topic.topic_replies,
                       'update_flag':    update_flag})
    return log_list
    
  # Function:zfProcessConfiguration():Saves system-wide (forum-wide) configuration information
  security.declareProtected('Admin zForums', 'zfProcessConfiguration')
  def zfProcessConfiguration(self):
    """ Saves system-wide (forum-wide) configuration information """
    req = self.REQUEST
    properties = self.zfGetSystemProperties()
    for property in properties:
      self.zfPutSystemProperty(property.property_name, req.get(property.property_name, ''))

    return req.RESPONSE.redirect('%s/config_html?updated=1' % (self.zAdmin.absolute_url()))

  # Function:zfCountTopicsInForum(forum_id):Returns the number of topics in a specific forum
  security.declareProtected('Admin zForums', 'zfCountTopicsInForum')
  def zfCountTopicsInForum(self, forum_id):
    return self.zSQL.getTopicsInForumCount(forum_id=forum_id)[0].topic_count

  # Function:zfGetAllForums():Returns all the forums regardless of the categories
  security.declareProtected('Admin zForums', 'zfGetAllForums')
  def zfGetAllForums(self):
    return self.zSQL.getAllForums()

  # Function:zfProcessCategories():Handles the request for adding/deleting categories
  security.declareProtected('Admin zForums', 'zfProcessCategories')
  def zfProcessCategories(self):
    """ Handles the request for adding/deleting categories """
    req = self.REQUEST
    add = req.get('add', '')
    remove = req.get('remove', '')
    move_up = req.get('up', '')
    move_down = req.get('down', '')

    if add:
      return req.RESPONSE.redirect('%s/add_category_form' % (self.zAdmin.absolute_url()))
    elif remove:
      ids = req.get('ids', [])
      deleted = 0
      if ids:
        for cat_id in ids:
          self.zSQL.delCategory(cat_id=cat_id)
          deleted += 1
      return req.RESPONSE.redirect('%s/categories_html?deleted=%s' % (self.zAdmin.absolute_url(), deleted))
    elif move_up:
      self.zfChangeCategoryPosition(move_up, 'up')
      return req.RESPONSE.redirect('%s/categories_html' % (self.zAdmin.absolute_url()))
    elif move_down:
      self.zfChangeCategoryPosition(move_down, 'down')
      return req.RESPONSE.redirect('%s/categories_html' % (self.zAdmin.absolute_url()))
    else:
      cat_id = req.get('cat_id', '')
      return req.RESPONSE.redirect('%s/categories_html' % (self.zAdmin.absolute_url()))

  # Function:zfGetForumsByCat(cat_id):Returns the forums that belong to a category
  security.declareProtected('Admin zForums', 'zfGetForumsByCat')
  def zfGetForumsByCat(self, cat_id):
    """ Returns the forums that belong to a category """
    forums = self.zSQL.getForumsByCat(cat_id=cat_id)
    if forums:
      all_forums = forums
    else:
      all_forums = {}
    return all_forums

  # Function:zfGetMembers():Returns a protected member list
  security.declareProtected('Admin zForums', 'zfGetMembers')
  def zfGetMembers(self):
    """ Returns a dictionary of important information about members """
    # Instead of a specific call for each property, it'll be much
    # less expensive to grab them all based on the acl_user and do some
    # dictionary'ing in the returned record, x/5 number of queries less.. :(
    users = self.zfGetUserNames()
    users.sort()
    user_list = []

    for user in users:
      user_list.append({'username': user,
                        'zfmp_username'     : self.zfGetUserProperty('zfmp_username', user, ''),
                        'zfmp_email'        : self.zfGetUserProperty('zfmp_email', user, ''),
                        'zfmp_last_login'   : self.zfGetUserProperty('zfmp_last_login', user, ''),
                        'zfmp_last_login_ip': self.zfGetUserProperty('zfmp_last_login_ip', user, ''),
                        'zfmp_web_page'     : self.zfGetUserProperty('zfmp_web_page', user, '')})

    return user_list
  
  # Function:zfMailPassword():Sends an email containing the password to the user
  security.declarePublic('zfMailPassword')
  def zfMailPassword(self):
    """ Handles Mail Password sending to an email """
    req = self.REQUEST
    srch_email = req.get('srch_email', '')
    errors = []
    if srch_email == '':
      errors.append('Missing Email Address, Please Try Again')
    else:
      this_user = self.zSQL.searchUserByEmail(email=srch_email)
      if this_user:

        # Email Password Functionality Here
        username = this_user[0].acl_user
        mfrom = self.zfGetSystemProperty('zfsp_admin_contact_email', 'admin@zforum.org')
        subject = "%s" % self.tr('global_newpwdreq', 'New Password Request')
        new_pwd = gen_pwd()
        # Body should probably be some kind of dynamic, admin-modificable text
        body = """Hello,
              
You recently requested a password retrieval from %s.

Your user name is: %s
Your temporary password is: %s

Please Log In the system using the password provided, then you can just click on the = My Preferences = button to change it to your preference.

To Log In, Go To:

%s/index_html

Or, to log in directly, go to:

%s/zfLogin?__ac_name=%s&__ac_password=%s""" % (self.zfForumBase(), username, new_pwd, self.zfForumBase(),
                                               self.zfForumBase(), username, new_pwd)
        mail_code = self.zfSimpleSendMessage(mto=srch_email, mfrom=mfrom, subject=subject, body=body)
        if mail_code == 0:
          user_roles = self.acl_users.getUser(username).getRoles()
          roles = []
          for rol in user_roles:
            roles.append(rol)
          # Change the password
          user_info = {'name': username,
                       'password': new_pwd,
                       'confirm': new_pwd,
                       'roles': roles}
          self.acl_users.manage_users(submit='Change', REQUEST=user_info)
        else:
          errors.append('%s %s' % (self.tr('err_mailcode', 'An error has occured while processing the request. Mail code:'), mail_code))
      else:
        errors.append(self.tr('err_emailnotfound', 'This email does not appear to be in our Database'))
    if errors:
      req.set('errors', errors)
      return self.login_form(self, REQUEST=req)
    else:
      return req.RESPONSE.redirect('%s/login_form?mail_code=%s' % (self.absolute_url(), mail_code))

  # ALL THE METHODS BELOW ARE NOT YET 100% VALIDATED AND MIGHT BREAK IF USED

  security.declarePublic('zfSmartSort')
  def zfSmartSort(self, list, key):
    nlist = map(lambda x, key=key: (x[key], x), list)
    nlist.sort()
    return map(lambda (key, x): x, nlist)
  security.declareProtected('Admin zForums', 'zfMakeAdmin')
  def zfMakeAdmin(self, username):
    """ Converts a Member to Admin """
    if username != 'administrator':
      user_info = {'name': username,
                   'password': 'password',
                   'confirm': 'pconfirm',
                   'roles': ['Reviewer']}
      self.acl_users.manage_users(submit='Change', REQUEST=user_info)
    return self.REQUEST.RESPONSE.redirect('%s/admin/admin_users_form' % (self.absolute_url()))

  security.declareProtected('Admin zForums', 'zfMakeMember')
  def zfMakeMember(self, username):
    """ Converts a Member to Admin """
    if username != 'administrator':
      user_info = {'name': username,
                   'password': 'password',
                   'confirm': 'pconfirm',
                   'roles': ['Member']}
      self.acl_users.manage_users(submit='Change', REQUEST=user_info)
    return self.REQUEST.RESPONSE.redirect('%s/admin/admin_users_form' % (self.absolute_url()))

  # Function:zfUserDel():Removes Users from zope, and cleans up the user's properties, leaves all forums and topics intact
  security.declareProtected('Admin zForums', 'zfUserDel')
  def zfUserDel(self):
    """ Handles Deletion of Users """
    req = self.REQUEST
    users   = req.get('ids', [])
    err = 0

    if users:
      # acl_users handling
      try:
        req.set('names', users)
        self.acl_users.manage_users(submit='Delete', REQUEST=req)
      except:
        err = 1
        pass

      for user in users:
        # SQL Handling
        self.zSQL.delUser(username=user)

    if err:
      deleted = 0
    else:
      deleted = len(users)
    return req.RESPONSE.redirect('%s/users_html?deleted=%s' % (self.zAdmin.absolute_url(), deleted))

  # Function:zfGrabUser(username):Grabs a Zope User
  security.declareProtected('Admin zForums', 'zfGrabUser')
  def zfGrabUser(self, username):
    return self.acl_users.getUser(username)

  # Function:feed_rss(section):Generates a valid RSS feed for zForum
  security.declarePublic('feed_rss')
  def feed_rss(self, section):
    """ RSS Feeder """
    response = self.REQUEST.RESPONSE
    friendly_urls = self.zfGetSystemProperty('zfsp_friendly_url', '')
    if section == 'latest':
      latest_topics = self.zSQL.getLatestTopics(max_announcements=20)
      if latest_topics:
        items = []
        for topic in latest_topics:
          modifying_date = topic.modifying_date
          year  = modifying_date.year()
          month = modifying_date.month()
          day   = modifying_date.day()
          hour  = modifying_date.hour()
          min   = modifying_date.minute()
          if friendly_urls:
            url = '%s/view_topic/%s/0' % (self.zfForumBase(), topic.topic_id)
          else:
            url = '%s/view_topic?topic_id=%s' % (self.zfForumBase(), topic.topic_id)
          item = PyRSS2Gen.RSSItem(
            title = self.zfParse(topic.topic_title),
            link = url,
            description = "%s" % (self.zfParse(topic.content, preview=1)),
            guid = PyRSS2Gen.Guid(url),
            pubDate = datetime.datetime(year, month, day, hour, min))
          items.append(item)
          
        if friendly_urls:
          url = "%s/feed_rss/%s" % (self.zfForumBase(), section)
        else:
          url = "%s/feed_rss?section=%s" % (self.zfForumBase(), section) 
        rss = PyRSS2Gen.RSS2(
          title = "Latest zForum Topics",
          link = url,
          description = "zForum is a SQL-Based Message Board Based in Zope",
          lastBuildDate = datetime.datetime.now(),
          items = items)
        response.setHeader('Content-type', 'text/xml')
        return rss.to_xml()
    elif section == 'system':
      system_announcements = self.zSQL.getSystemAnnouncements(max_announcements=20)
      if system_announcements:
        items = []
        for topic in system_announcements:
          modifying_date = topic.modifying_date
          year  = modifying_date.year()
          month = modifying_date.month()
          day   = modifying_date.day()
          hour  = modifying_date.hour()
          min   = modifying_date.minute()
          if friendly_urls:
            url = '%s/view_topic/%s/0' % (self.zfForumBase(), topic.topic_id)
          else:
            url = '%s/view_topic?topic_id=%s' % (self.zfForumBase(), topic.topic_id)
          item = PyRSS2Gen.RSSItem(
            title = self.zfParse(topic.topic_title),
            link = url,
            description = "%s" % (self.zfParse(topic.content, preview=1)),
            guid = PyRSS2Gen.Guid(url),
            pubDate = datetime.datetime(year, month, day, hour, min))
          items.append(item)
          
        if friendly_urls:
          url = "%s/feed_rss/%s" % (self.zfForumBase(), section)
        else:
          url = "%s/feed_rss?section=%s" % (self.zfForumBase(), section)
        rss = PyRSS2Gen.RSS2(
          title = "Latest zForum Topics - System Announcements",
          link = url,
          description = "zForum is a SQL-Based Message Board Based in Zope",
          lastBuildDate = datetime.datetime.now(),
          items = items)
        response.setHeader('Content-type', 'text/xml')
        return rss.to_xml()
    pass


def oSmartSort(data, order):

  def compare(x, y):
    attribute = 'modifying_date'
    return cmp(getattr(x, attribute), getattr(y, attribute))

  data.sort(compare)

  if order != 'A':
    data.reverse()

  return data



# Taken From ${ZOPE_DIR}/lib/python/OFS/Image.py
def getImageInfo(data):
  data = str(data)
  size = len(data)
  height = -1
  width = -1
  content_type = ''

  # handle GIFs
  if (size >= 10) and data[:6] in ('GIF87a', 'GIF89a'):
    # Check to see if content_type is correct
    content_type = 'image/gif'
    w, h = struct.unpack("<HH", data[6:10])
    width = int(w)
    height = int(h)

  # See PNG v1.2 spec (http://www.cdrom.com/pub/png/spec/)
  # Bytes 0-7 are below, 4-byte chunk length, then 'IHDR'
  # and finally the 4-byte width, height
  elif ((size >= 24) and (data[:8] == '\211PNG\r\n\032\n')
      and (data[12:16] == 'IHDR')):
    content_type = 'image/png'
    w, h = struct.unpack(">LL", data[16:24])
    width = int(w)
    height = int(h)
 
  # Maybe this is for an older PNG version.
  elif (size >= 16) and (data[:8] == '\211PNG\r\n\032\n'):
    # Check to see if we have the right content type
    content_type = 'image/png'
    w, h = struct.unpack(">LL", data[8:16])
    width = int(w)
    height = int(h)

  elif (size >= 2) and (data[:2] == '\377\330'):
    content_type = 'image/jpeg'
    jpeg = StringIO(data)
    jpeg.read(2)
    b = jpeg.read(1)
    try:
      while (b and ord(b) != 0xDA):
        while (ord(b) != 0xFF): b = jpeg.read(1)
        while (ord(b) == 0xFF): b = jpeg.read(1)
        if (ord(b) >= 0xC0 and ord(b) <= 0xC3):
          jpeg.read(3)
          h, w = struct.unpack(">HH", jpeg.read(4))
          break
        else:
          jpeg.read(int(struct.unpack(">H", jpeg.read(2))[0])-2)
        b = jpeg.read(1)
      width = int(w)
      height = int(h)
    except: pass

  return content_type, width, height

# One of the most critical, Protecting what needs to be protected,
# most likely this will be ther "admin" section and some "sql" queries
def changePermissions(self, id):
  """Manage Permissions for specific published objects"""
  
  zforum = getattr(self, id)
  
  # Some Folder Protections:
  # Admin Section Must be readable by Admins/Managers only
  zforum.zAdmin.manage_permission('Access contents information',('Manager', 'zAdministrator', 'zAdministratorStaff',),0)
  zforum.zAdmin.manage_permission('View',('Manager', 'zAdministrator', 'zAdministratorStaff',),0)
  # No SQL methods will be accessed directly from ZPT/DTML, all goes throuth the zForum API
  zforum.zSQL.manage_permission('View',('Manager', 'zAdministrator', 'zAdministratorStaff',),0)
   
  pass

def setForumRoles(self, id, plone_instance):
  """ Manage Forum Roles - Plone Aware """

  zforum = getattr(self, id)

  if plone_instance:
    acq_wrapper = zforum.aq_inner.aq_parent
    # acq_wrapper now contains the zForum's parent container.. but
    # I need to make sure that is a plone site..
    if acq_wrapper.meta_type == 'Plone Site':
      role_container = acq_wrapper
    else:
      role_container = zforum
  else:
    role_container = zforum

  for role in ['zAdministrator', 'zAdministratorStaff', 'zModerator', 'zMember', 'zMemberPremiere']:
    role_container._addRole(role=role)
    
  pass

InitializeClass(zForum)
